
--	BurkeLib.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.3
--	CREATED:	01-05-2011 --3dsmax 2011
--	MODIFIED:	04-29-2011 --3dsmax 2011


--	DESCRIPTION:
--	Library of general-purpose functions


--	REVISION history:
--	1.3 Cleaned Objects always have a scale of [1,1,1]
--  1.2 fixed link problem with Object Cleaner
--	1.1	getTheLastElementNameInFileAddress -ADDED
--	1.0	giveMeAGridOfPositions -ADDED
--------------------------------------------------------

	global BurkeLibLoaded = true
	global automat_CleanMessage
	global returnThisImageFromAutomat
	
	--------------------------------------------------------
	--This is the number of icons in the bitmap file BURKE-Icons2 which contains all
	--of the standard-sized icons used in the Automat interface
	local Icons
	local IconsInFile = 16
	local openBurkeIconsWithProperGamma
	local openAnyOldImageWithProperGamma
	local currentMaxVersion = (maxVersion())[1]/1000.0
	
	--------------------------------------------------------
	
	local SetThisFaceMatID = polyop.setFaceMatID
	local setMaterialIDForSelection
	local getMaterialIDForSelection
	local GetFaceDeg = polyop.getFaceDeg
	local VertPos = polyop.getVert
	--------------------------------------------------------
	local giveMeAGridOfPositions
	local cleanSelectionOfObjects
	--------------------------------------------------------
	local GridOfPositions=#()
	local GridSize=#(0,0,0) --X,Y,Z dimensions
	local GridStart=#(0,0,0) --X,Y,Z dimensions
	local GridSpacing = 10.0 
	--------------------------------------------------------
	
	
	--------------------------------------------------------
	
	function openBurkeIconsWithProperGamma =
	(
			
		--max 2013 and greater
		if currentMaxVersion > 14 then
		(
			--Use this method for max 2013 and above
			tempopen = OpenBitMap  (  GetDir #userIcons + "\\BURKE-Icons2_24i.bmp" )
			Icons = bitmap tempopen.width tempopen.height
			copy tempopen Icons
		)
		
		else
		-- Use this method for older versions of max
		(
			--load the icons with correct gamma
			tempingamma = fileInGamma
			fileInGamma = 1
			tempopen = OpenBitMap  (  GetDir #userIcons + "\\BURKE-Icons2_24i.bmp" )
			Icons = bitmap tempopen.width tempopen.height
			copy tempopen Icons
			--Icons.gamma = tempingamma
			fileInGamma = tempingamma
		)
	)
	
	--------------------------------------------------------
	
	function openAnyOldImageWithProperGamma getThisImage =
	(
			
		--max 2013 and greater
		if currentMaxVersion > 14 then
		(
			--Use this method for max 2013 and above
			tempopen = OpenBitMap (  getThisImage )
			returnThisImageFromAutomat = bitmap tempopen.width tempopen.height
			copy tempopen returnThisImageFromAutomat
		)
		
		else
		-- Use this method for older versions of max
		(
			--load the icons with correct gamma
			tempingamma = fileInGamma
			fileInGamma = 1
			tempopen = OpenBitMap (  getThisImage )
			returnThisImageFromAutomat = bitmap tempopen.width tempopen.height
			copy tempopen returnThisImageFromAutomat
			fileInGamma = tempingamma
		)
	)
	
	--------------------------------------------------------
	
	function giveMeAGridOfPositions  GridSize GridSpacing =
		(
			GridOfPositions=#() --RESET ARRAY
			
			
			for AxisLoop = 1 to 3 do --START POSITION X,Y,Z
			(
				a = GridSize[AxisLoop]
				GridStart[AxisLoop] =  0 -  ( ( a - 1 ) * (GridSpacing/2 ) )
			)
			
			PositionCount = GridSize[1] * GridSize[2] * GridSize[3] --POSITIONS IN GRID
			for piggy = 1 to PositionCount do append GridOfPositions [0,0,0] --SET UP ARRAY WITH POINT3
			
			BigLoop = 1
			
			for ZLoop = 1 to GridSize[3] do
				(	
					for YLoop = 1 to GridSize[2] do
					(
						for XLoop = 1 to GridSize[1] do
						(
							CurrentX = GridStart[1] + GridSpacing * (XLoop - 1)
							CurrentY = GridStart[2] + GridSpacing * (YLoop - 1)
							CurrentZ = GridStart[3] + GridSpacing * (ZLoop - 1)
							
							GridOfPositions[BigLoop] = [ CurrentX, CurrentY, CurrentZ ]
							Bigloop += 1
						)
					)
				)
			
			--format "GridOfPositions: %\n\n" GridOfPositions
		)
		
		--------------------------------------------------------
		local WorkingString
		
		--------------------------------------------------------
		function getTheLastElementNameInFileAddress WorkingString =
		(
			StringArray = filterstring WorkingString "\\"
			--format "StringArray: %\n" StringArray
			WorkingString = StringArray[StringArray.count]
			
		)
		
		--------------------------------------------------------
		local ObjectCount = 0
		
		
		--------------------------------------------------------
		function findGridSizeAndSpacing  ObjectCount SpreadLength =
		(
			if ObjectCount == 0 then GridSize=#(0,0,0)

				if ObjectCount < 4 then GridSize=#(ObjectCount,1,1)
					else
					(
						CountRoot = ( ceil ( sqrt ObjectCount) ) as integer
						if ObjectCount <= CountRoot*(CountRoot-1)  then GridSize=#(CountRoot,CountRoot-1,1)
							else GridSize=#(CountRoot,CountRoot,1)
				
					)
				local DivideBy
				if 1 == GridSize[1] then DivideBy = 1 else DivideBy = ( GridSize[1] - 1 )
				GridSpacing = SpreadLength / DivideBy 	--SETS AS DIVISION OF X AXIS FOR NOW
		)
		
		--------------------------------------------------------
		function gimmeCommas GivenNum =
		(
			numberasstring = GivenNum as string
			digitcount = numberasstring.count
			
			if digitcount >= 4 then
			(
				commacount = 1 --reset
				while 1 + ( 3 * ( commacount +1) ) <= digitcount do commacount += 1
		
				docomma = digitcount - ( 3 * commacount )
				
				buildastring = ""
				
				for loop = 1 to digitcount do
				(
					append buildastring numberasstring[loop]
					if loop == docomma and loop < digitcount do
					(
						append buildastring ","
						docomma += 3
					)
				)
				GivenNum = buildastring
			)
			else GivenNum as string --return as a string
		)
		
		--------------------------------------------------------
		function setMaterialIDForSelection ObjArray bool_DoRandom MatSmoothNumSelector =
		(
			local wasmode
			local waslevel
			local modifynotopen = (getCommandPanelTaskMode() != #modify)
			
			with redraw off
			loopend = objArray.count

			
			if modifynotopen do
			(
				wasmode = getCommandPanelTaskMode()
				waslevel = subObjectLevel
				max modify mode
			)
					
			if 1 == loopend and subObjectLevel != 0 then
				(
					--single object
					CurrentObj = objArray[1]      -- was .baseobject
					
					currentthing = modPanel.getCurrentObject()
					
					if Edit_Poly == classof( currentthing ) then
					(	
						
						--Edit Poly	
						--don't need this   local selectarray = currentthing.getSelection #face
						currentthing.SetOperation #SetMaterial
						currentthing.MaterialIDtoSet = ( MatSmoothNumSelector - 1 )	
						currentthing.Commit ()

					)
					
					else if Editable_Poly == classof( currentthing ) do
					(	

						--Single EditablePoly
						currentthing = CurrentObj.baseobject
						tempForTesting = polyop.getFaceSelection  currentthing
						SetThisFaceMatID currentthing tempForTesting MatSmoothNumSelector
						
						modPanel.setCurrentObject currentthing
						
					)

				)
				else --more than one object
				( 	
					for i = 1 to loopend do
					(
						CurrentObj = ObjArray[i].baseobject
						if Editable_Poly == classof(CurrentObj) do
						(
							
							wiggly = polyop.getnumfaces CurrentObj
							for faceloop =1 to wiggly do SetThisFaceMatID CurrentObj faceloop MatSmoothNumSelector
							
						)
					)
				)	
				
				
				if modifynotopen do
				(
					setCommandPanelTaskMode wasmode
					if ( undefined != waslevel) do subObjectLevel = waslevel
				)
				
				
		)
		

	
	--------------------------------------------------------
	function getMaterialIDForSelection objArray bool_DoRandom MatSmoothNumSelector =
	(
		
			local wasmode
			local waslevel
			local modifynotopen = (getCommandPanelTaskMode() != #modify)
		
			with redraw off
			loopend = objArray.count
		
			if modifynotopen do
			(
				wasmode = getCommandPanelTaskMode()
				waslevel = subObjectLevel
				max modify mode
			)
	
			for i = 1 to loopend do
				
			(	
				CurrentObj = ObjArray[i]
				baseof = CurrentObj.baseobject
				local foundeditpoly = false
				
				currentthing = modPanel.getCurrentObject()
				
				if Edit_Poly == classof( currentthing ) then foundeditpoly = true
				else 
				(
					editpolymods = for each in CurrentObj.modifiers where classOf each == Edit_Poly collect each
					if editpolymods.count >0 then 
					(
						currentthing = editpolymods[1]
						foundeditpoly = true
					)
				)
				
				if true == foundeditpoly then
				(
					print "Doing Edit_Poly Method\n"
					
					--ignore Edit Poly
				
					
				)
				
				else if Classof( currentthing ) == Editable_Poly then
				(
					
					print "Doing Editable_Poly Method\n"
					
					AllTheFaces=polyop.getnumfaces baseof --how many faces in current object
					local FaceSelectionArray=#{} --empty the bit arrray gonna use to collect faces that match the ID am looking for
				
					for CurrentCounter=1 to AllTheFaces do
					(
						if polyOp.getFaceMatID baseof CurrentCounter == MatSmoothNumSelector do
						(
							append FaceSelectionArray CurrentCounter
						)
					)
					polyop.setFaceSelection baseof FaceSelectionArray
				
				)
				
				
			)
			
				
			
			if modifynotopen do
			(
				setCommandPanelTaskMode wasmode
				if ( undefined != waslevel) do subObjectLevel = waslevel
			)
			
	)
	
	
-- 	--------------------------------------------------------
-- 	function getMaterialIDForSelection objArray bool_DoRandom MatSmoothNumSelector =
-- 	(
-- 		for i = 1 to objArray.count do
-- 			
-- 			(	
-- 				CurrentObj = ObjArray[i].baseobject
-- 				AllTheFaces=polyop.getnumfaces CurrentObj --how many faces in current object
-- 				local FaceSelectionArray=#{} --empty the bit arrray gonna use to collect faces that match the ID am looking for
-- 			
-- 				for CurrentCounter=1 to AllTheFaces do
-- 				(
-- 					if polyOp.getFaceMatID CurrentObj CurrentCounter == MatSmoothNumSelector do
-- 					(
-- 						append FaceSelectionArray CurrentCounter
-- 					)
-- 				)
-- 				polyop.setFaceSelection CurrentObj FaceSelectionArray
-- 			)
-- 	)
	
	--------------------------------------------------------
	function cleanSelectionOfObjects Selection ObjCleanPrefs =
	(
		with redraw off
		(
			with undo off
			(
				max create mode
				
				CleanObjArray = for piggy in selection where superclassof(piggy) == GeometryClass collect piggy
				MisCleanObjArray = for piggy in selection where superclassof(piggy) != GeometryClass collect piggy
				
				--------------
				GoomahCleanedSelection=#() --empty out this temporary array used for accumulating objects processed
				
				--------------
				if CleanObjArray.count > 0 then
				(					
					for EachObject in CleanObjArray do
					(
						--pivot
						--------------
						tempbox = convertToPoly( box position: (EachObject.pivot) name: (EachObject.name as string) ) 
						
						--layers
						--------------
						if true == ObjCleanPrefs.layer do
						(
							currentlayer = layermanager.getlayerfromname( EachObject.layer.name )
							currentlayer.addnode tempbox
						)
						
						--Backface Culling
						--------------
						if true == ObjCleanPrefs.backcull do
						(
							tempbox.backFaceCull  = EachObject.backFaceCull
						)
						
						--Wire Color
						--------------
						if true == ObjCleanPrefs.wire do
						(
							tempbox.wirecolor  = EachObject.wirecolor
						)
						
						--alignment
						--------------
						if true == ObjCleanPrefs.pivotmode do
						(
							tempbox.parent = EachObject
							AlignToParent tempbox
							tempbox.parent = undefined
							tempbox.scale = [ 1, 1, 1 ] --RESET SCALE
						)
						
						--parent
						--------------
						if true == ObjCleanPrefs.parent do
						(
							if undefined != EachObject.parent do tempbox.parent = EachObject.parent
						)
						
						--child
						--------------
						if true == ObjCleanPrefs.child do
						(
							numchildren = EachObject.children.count
							if numchildren > 0 do
							(
								for loop = numchildren to 1 by -1 do append tempbox.children EachObject.children[loop] 
							)								
						)
						
						--Out with the old
						--------------------------------------------------------
						polyop.deleteFaces tempbox  #{1..tempbox.GetNumFaces()}
						tempbox.attach EachObject tempbox  	--ataches OriginalObject to TempBox using TempBox XForm
						
						--material
						--------------
						if false == ObjCleanPrefs.mats do
						(
							tempbox.material = undefined
						)
						
-- 						if Check_CleanedMat.state == true do Assignmat ($)
					
						append GoomahCleanedSelection tempbox --add each item cleaned to cleaned item array	
					)
-- 					
-- 					--------------
					CleanedObjectsCount = GoomahCleanedSelection.count
					
					select GoomahCleanedSelection
					
					automat_CleanMessage = if 0 == CleanedObjectsCount then "NO Objects Cleaned, "
						else if 1 == CleanedObjectsCount then "1 Cleaned, "
							else CleanedObjectsCount as string +  " Cleaned, "
					automat_CleanMessage = automat_CleanMessage + ( MisCleanObjArray.count as string ) + " Skipped."

				)
				else ( automat_CleanMessage = "Nothing to clean" ) --no cleanable objects found
				
			) --undo
		) --redraw
	) --function
	
	
	