
--	automat_ImportExportTools.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.0
--	CREATED:	01-27-2012 --3dsmax 2012


--01-27-12	Breaking these scripts out from Object Tools. added directory copy function. moved pref saving into update function.
(
	--this is the rollout name
	global automat_ImportExportTools
	--
	--------------------------------------------------------
	local CurrentINI = GetDir #plugcfg + "\\automat_ImportExportTools.ini"
	local DirectorySpecificINI --to be written and read for pivot information
	--------------------------------------------------------
	if true != BurkeLibLoaded do include "$userScripts/BurkeStudios/lib/BurkeLib.ms"

	
	
	--Function Names
	--------------------------------------------------------

	local exportSelectionToObj
	local storePivotsForExportObjects
	local updateExportDirText
	
	local importOBJDir

	--
	local ExpObj_FilePath = (  (getdir #export)+"\\" ) --default location
	local ImpObj_FilePath = (  (getdir #import)+"\\" ) --default location
	
	tipa= "When you export objects from this tool. The pivots are stored in an ini file in the same directory. If the base filenames do not change then this tool can restore pivots that would otherwise be lost with OBJ import/export."
	tipb="\n\nFor the best results use only upper and lowercase letters, numbers, and the underscore character."
	tipc = tipa + tipb
	
	--UI
	--/////////////////////////////////////////////
	--------------------------------------------------------
	rollout roll_automat_ImportExportTools "Import/Export Tools"
	(
		group "IMPORT Directory:"
		(
			
			button but_ImportOBJs "Import Directory" height:24 tooltip:"Import all files in specified directory with the selected characteristics below."
			--
			--edittext text_Suffix "ALL:" text:"-uvlayout.obj"  offset:[0,0]
			combobox text_Suffix "Load all files that match:" items:#("*.obj","*-uvlayout.obj")  height: 4 offset:[0,0]
			button but_location2 "Location..." width:72 across:2 offset:[-3,4] tooltip:"Choose a new location."
			button but_matchExp "Match Exp." width:67 offset:[6,4] enabled:true tooltip:"Set location to be the same as Export Directory."
			edittext text_ImportDir "" text:"...\max\import\\" readonly:true width:146 offset:[-10,-5]

			checkbox check_restorePivots "Restore Pivots" offset:[0,2] checked:false \
			tooltip: "When you export OBJ files using this tool, object pivot data is written to an ini file in the same directory.\n
If the object names stay consistent, pivots can be restored.

Max's OBJ importer, however, will change object names if you use characters outside of letters, numbers, and the underscore. This WILL break the ability to restore pivots.\n
The only exception to this is '-uvlayout' which the script handles as a special case."
			checkbox check_RenameObjects "Rename by filename" offset:[0,0] checked:false tooltip:"Rename imported objects based on filename. \ni.e. All objects imprted from Gecko.obj would be named Gecko_000, Gecko_001, etc."
		)
		--
		
		--
		group "EXPORT OBJs:"
		(
			
			button but_ExportOBJs "Export Selected" height:24 tooltip:"Export each object in selection as a separate OBJ file.\n\nYou will have a chance to adjust the export settings for the first object and the rest of the objects will use those settings."
			button but_location "Location..." width:72 across:2 offset:[-3,4] tooltip:"Choose a new location."
			button but_matchImp "Match Imp." width:67 offset:[6,4] enabled:true tooltip:"Set location to be the same as Import Directory."
			edittext text_ExportDir "" text:"...\max\export\\" readonly:true width:146 offset:[-10,-5]
		)
		
		
		--/////////////////////////////////////////////
		--OPEN
		--/////////////////////////////////////////////
		--------------------------------------------------------
		on roll_automat_ImportExportTools open do
		(
			
			getapref = ( getINISetting CurrentINI "Export to OBJ" "Directory Location" ) as string
			if getapref != "" do ExpObj_FilePath = getapref
			--
			getapref = ( getINISetting CurrentINI "Import Directory" "Directory Location" ) as string
			if getapref != "" do ImpObj_FilePath = getapref
			--
			getapref = execute ( getINISetting CurrentINI "Import Settings" "Rename by Filename" )
			if getapref != OK do check_RenameObjects.checked = getapref
			--
			getapref = execute ( getINISetting CurrentINI "Import Settings" "Restore Pivots" )
			if getapref != OK do check_restorePivots.checked = getapref
			--
			updateExportDirText()
		)
		--
		on check_restorePivots changed state do
		(
			setapref =  setINISetting CurrentINI "Import Settings" "Restore Pivots" (state as string )
		)
		--
		on check_RenameObjects changed state do
		(
			setapref =  setINISetting CurrentINI "Import Settings" "Rename by Filename" (state as string )
		)
		
		--BUTTONS
		--/////////////////////////////////////////////
		--------------------------------------------------------
		
		on but_location pressed do
		(
			beebee=undefined
			beebee = getSavePath initialDir:ExpObj_FilePath historyCategory:"automat_ExportToOBJ" \
				caption:"Select a directory for saving exported OBJ files." 
			
			if undefined != beebee do
			(
				ExpObj_FilePath = beebee + "\\"
				--------------

				
				--------------
			)
			--
			updateExportDirText()
		)
		--
		on but_location2 pressed do
		(
			beebee=undefined
			beebee = getSavePath initialDir:ExpObj_FilePath historyCategory:"automat_ImportOBJ" \
				caption:"Select a directory for loading OBJ files." 
			
			if undefined != beebee do
			(
				ImpObj_FilePath = beebee + "\\"
				--------------
				
				--------------
			)
			--
			updateExportDirText()
		)
		
		--------------------------------------------------------
		--------------------------------------------------------


		--------------
		on but_ExportOBJs pressed do
		(
			exportSelectionToObj selection
		)
		
		on but_ImportOBJs pressed do
		(
			importOBJDir()
		)
		
		
		on but_matchImp pressed do
		(
			ExpObj_FilePath = ImpObj_FilePath
			updateExportDirText()
		)
		
		
		on but_matchExp pressed do
		(
			ImpObj_FilePath = ExpObj_FilePath
			updateExportDirText()
		)
		
		
		
	)--rollout
	
	--FUNCTIONS
	--/////////////////////////////////////////////
	--------------------------------------------------------

	
	--------------------------------------------------------

	
	--------------------------------------------------------

	
	--------------------------------------------------------
	fn exportSelectionToObj selection =
	(
		
		--first export pivots for later
		--will add ini functionality later, safer
		storePivotsForExportObjects()
		
		--
		
		ObjArray = for piggy in selection where superclassof(piggy) == GeometryClass collect piggy
		local ExportLoop = ObjArray.count
		local ExportCount = 0
		--------------
		--export first file to set dialog properties
		if ExportLoop > 0 do filename = ExpObj_FilePath  + ObjArray[1].name + ".obj"
		select ObjArray[1]
		DidSave = exportFile filename  selectedOnly: true --#noPrompt
		if DidSave do ExportCount += 1
		--------------
		if ExportLoop > 1 and DidSave do for loop = 2 to ExportLoop do
		(
			select ObjArray[loop]
			filename = ExpObj_FilePath  + ObjArray[loop].name + ".obj"
			DidSave = exportFile filename  #noPrompt selectedOnly:true 
			if DidSave do ExportCount += 1
		)
		
		select ObjArray --restore all objects actually saved
		
		if 0 == ExportCount then messagebox "No objects exported."
			else if 1 == ExportCount then messagebox "1 File exported to OBJ." 
				else messagebox ( ( ExportCount as string ) + " Files exported to OBJ." )
		
	)--END Export to OBJ
	
	--------------------------------------------------------
	function importOBJDir =
	(
		OBJsInDir = getFiles ( ImpObj_FilePath + "\*" + roll_automat_ImportExportTools.text_Suffix.text )
		
		loopend = OBJsInDir.count
		for loop = 1 to loopend do
		(
			if true == keyboard.escPressed do return()
			
			--
			importFile OBJsInDir[loop] #noPrompt
			
			--reset pivots if needed
			
			
			if roll_automat_ImportExportTools.check_restorePivots.checked do
			(
				DirectorySpecificINI =  ( (  ImpObj_FilePath )  +  "_objectpivotdata.ini" )
				
				for each in selection do
				(
					getapref = execute ( getINISetting DirectorySpecificINI  each.name  "Pivot" )
						if getapref != OK do each.pivot = getapref
				)
			)--if
				
			--------------------------------------------------------
			--rename objects if needed
			if roll_automat_ImportExportTools.check_RenameObjects.checked do
			(
				local namescount = 0
				thisfilename = getTheLastElementNameInFileAddress OBJsInDir[loop]
				trimname = substring thisfilename 1 (thisfilename.count - 4) --trim the file extension off the end
				
				for each in selection do
				(
					if 0 == namescount then each.name = trimname --works for first object in each file.
						else each.name = ( trimname + "_" + (namescount as string) )
					namescount += 1
				)
			)--if
			
			
		)--for loop

	)--fn
	
	function storePivotsForExportObjects =
	(
		--establish directory-specific ini file
		DirectorySpecificINI =  ( (  ExpObj_FilePath )  +  "_objectpivotdata.ini" )
		
			for each in selection where isValidNode(each) do 
			(
				setapref =  setINISetting DirectorySpecificINI  each.name  "Pivot"  ( each.pivot as string )
			)
			
	)
	
	--------------------------------------------------------
	fn updateExportDirText =
	(
		startofpath = substring ExpObj_FilePath 1 3
		LocationName = startofpath + " ...\\" + ( getTheLastElementNameInFileAddress ExpObj_FilePath )
		roll_automat_ImportExportTools.text_ExportDir.text = LocationName
		--
		startofpath = substring ImpObj_FilePath 1 3
		LocationName = startofpath + " ...\\" + ( getTheLastElementNameInFileAddress ImpObj_FilePath )
		roll_automat_ImportExportTools.text_ImportDir.text = LocationName
		
		setapref =  setINISetting CurrentINI "Export to OBJ" "Directory Location" (  "\"" + ExpObj_FilePath as string + "\"" )
		setapref =  setINISetting CurrentINI "Import Directory" "Directory Location" (  "\"" + ImpObj_FilePath as string + "\"" )
	)
	
	--------------------------------------------------------
	
	
	
	--------------------------------------------------------
	if true != automat_OpenScriptAsRollout do
	(
		try (closeRolloutfloater automat_ImportExportTools ) catch ()
		automat_ImportExportTools = newRolloutFloater "Import / Export Tools" 170 90
		addRollout roll_automat_ImportExportTools automat_ImportExportTools
		automat_ImportExportTools.size.y = ( roll_automat_ImportExportTools.height + 30 )
	)
	
)
