
--	MatSmoothTool.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.2
--	CREATED:	02-19-2011 --3dsmax 2011

--	4-2-2014	Added automagic new test material functionailty.
--	3-16 		removed random button. No point to it. Added Clear mats button which is at least useful.
--	1-6-2012	fixed bug where assigning mat IDs in element mode would apply those IDs to whole object and not just selected faces

(
	global automat_MatSmoothTool
	
	local CurrentINI = GetDir #plugcfg + "\\automat_MatSmoothTool.ini"
	
	if true != BurkeLibLoaded do include "$userScripts/BurkeStudios/lib/BurkeLib.ms" 
		
	local RolloutPosition= [200,200]
	
	getapref = execute ( getINISetting CurrentINI "MatSmooth" "RolloutPosition" )
			if getapref != OK do RolloutPosition = getapref
	--------------------------------------------------------
		
	
	--Function Names
	--------------------------------------------------------
	local setMaterialIDForSelection
	local getMaterialIDForSelection
	local setSmoothIDForSelection
	local getSmoothIDForSelection
	
	local ObjArray
	local MatSmoothNumSelector = 0
	local SetMatOrSmooth = "mat"
	local GetOrSet = "set"
	local SelectedFaces
	local GivenSelection
	local RandomNumber
	local ReplaceSmoothBool = false
	local bool_DoRandom = false
	local SmoothingGroupNumber = 1
	local MaterialsTooltips = "Left-click to assign.\nCtrl+Left-click to assign (11-20)\nShift+Ctrl+Left-click to assign (21-30)\n\n" + \
	"Right-click to select.\nCtrl+Right-click to select (11-20)\nShift+Ctrl+Right-click to select (21-30)"
	
	local automatmaterial, fallbackmaterial
	local u_tiling = 10, v_tiling = 10
	local automat_UVTiling_floater, automat_UVTiling_rollout
	
	
	
		--//////////////////////////////////////////////////////////////////////////////////////////////////////
		function makeNewAutomatMaterial u_tiling v_tiling =
		(
			--first the multimaterial
			
-- 			if okMtlForScene != true do
-- 			(
-- 				messagebox "already an Automat_TestMaterial in scene"
-- 			)
			automatmaterial = multimaterial name:"Automat_TestMaterial"
			prettycolorsarray = #(  (color 40 66 141), (color 9 38 150), (color 22 12 113), (color 86 42 212), (color 231 47 176), (color 245 78 0), (color 255 154 0), (color 64 126 0), (color 27 100 20), (color 25 119 82) )
			
			--creates new maps with proper u and v tiling
			for cunny = 1 to 10 do
			(
				
				tempcolor = black
				tempcolor.r = prettycolorsarray[cunny].r
				tempcolor.g = prettycolorsarray[cunny].g
				tempcolor.b = prettycolorsarray[cunny].b
				
				
				
				tempvalue = tempcolor.v
				if tempvalue> 40 then tempcolor.v -= 40 else tempcolor.v = 0
						
						
				tempmap = checker color1: prettycolorsarray[cunny] color2: tempcolor
				tempmap.coordinates.u_tile = true
				tempmap.coordinates.u_tiling = u_tiling
				tempmap.coordinates.v_tile = true
				tempmap.coordinates.v_tiling = v_tiling
			
				numastext = cunny as string
				padding = if numastext.count == 1 then "0" else ""
				tempname = "Automat Mat " + padding + numastext
			
				tempmat = standardMaterial name: tempname diffuseMap:tempmap
			

				automatmaterial[cunny] = tempmat
				showTextureMap automatmaterial[cunny] tempmap on
			)--end cunny
			
		) --end function
		
		function getUserInput_MakeMaterial  =
		(
			--gets input from user and makes new material
			
			try (closeRolloutfloater automat_UVTiling_floater ) catch ()
			rollout automat_UVTiling_rollout "Create New Test Material"
			(
				spinner spin_Utiling "U Tiling:" range:[1,600,10] type:#integer
				spinner spin_Vtiling "V Tiling:" range:[1,600,10] type:#integer
				button but_return "Make Test Materials" tooltip: "Press to create new materials with above settings"
				
				on but_return pressed do
				(
					u_spin = spin_Utiling.value
					
					v_spin = spin_Vtiling.value
					

					try (closeRolloutfloater automat_UVTiling_floater ) catch ()
					
					makeNewAutomatMaterial u_spin v_spin --make material
					
					$.material = automatmaterial

				)
				
			)--end rollout
			
			automat_UVTiling_floater = newRolloutFloater "Set UV Tiling Properties" 170 102
			addRollout automat_UVTiling_rollout automat_UVTiling_floater
			
		)--end function
	
	--UI
	--/////////////////////////////////////////////
	--------------------------------------------------------
	rollout roll_automat_MatSmoothTool "Material ID and Smoothing"
	(
		
		button but_MaterialID_01"1" tooltip:MaterialsTooltips width:15 pos:[2,4]
		button but_MaterialID_02"2" tooltip:MaterialsTooltips width:15 pos:[17,4]
		button but_MaterialID_03"3" tooltip:MaterialsTooltips width:15 pos:[32,4]
		button but_MaterialID_04"4" tooltip:MaterialsTooltips width:15 pos:[47,4]
		button but_MaterialID_05"5" tooltip:MaterialsTooltips width:15 pos:[62,4]
		button but_MaterialID_06"6" tooltip:MaterialsTooltips width:15 pos:[77,4]
		button but_MaterialID_07"7" tooltip:MaterialsTooltips width:15 pos:[92,4]
		button but_MaterialID_08"8" tooltip:MaterialsTooltips width:15 pos:[107,4]
		button but_MaterialID_09"9" tooltip:MaterialsTooltips width:15 pos:[122,4]
		button but_MaterialID_10"10" tooltip:MaterialsTooltips width:17 pos:[137,4]
		
		checkbutton but_MaterialMode "Material Mode" width:76 height:22 pos:[2,28] highlightColor:[130,150,180] checked:true\
		tooltip:"Assign/Select Material IDs.\n\nTool will assign material IDs to an EditPoly modifier or to an Editable Poly object, whichever is active. The selection function only works with Editable Poly objects.\n
Unless an Edit Poly modifier is active, all assignments are made to the base Editable Poly object.\n
If several objects are selected, the mat ID will be assigned to all faces in the basemesh of each object."
		
		checkbutton but_SmoothMode "Smooth Mode" width:76 height:22 pos:[78,28] highlightColor:[130,150,180]\
		tooltip:"Assign/Select Smoothing Categories.\n\nSmoothing features do not work with the EditPoly modifier.\n
With smooth groups, modifiers are ignored and smoothing will be assigned to the base objects. If you are editing a single EditablePoly object, smooth groups will be assigned to your current face selection.\n
If several objects are selected, the picked smooth group will be assigned to all faces in the basemesh of each object."
		--Radiobuttons rad_MatIDMode labels:#("Material ID" , "Smoothing") columns:2 pos:[2,28] 
		
		Checkbox Check_ReplaceSmooth "Replace Smoothing Mode" checked:true \
			tooltip:"Replaces existing smoothing information for selection."\
				pos:[4,54]
		
		--button but_MatSmoothlID_Random "Random Mat" alignment: #right offset:[-3,0] across:2 tooltip:"Assigns random material IDs (1-10) to object."
		button but_ClearMats "ClrMats"  width: 46 pos:[2,74]  tooltip:"Clear materials from selected objects." 
		button but_ClearSmoothing "ClrSmooth"  width: 60 pos:[48,74]  tooltip:"Clear smoothing from selected objects." 
		button but_ApplyTestMat "TestMat"  width: 47 pos:[108,74]  tooltip:"Click to create and apply an
Automat_TestMaterial to selection.
(Any existing copies of the test material
will be deleted.\n
Alt-Click to first look for an existing Automat_TestMaterial in scene and use that if found." 
		
		
		
	--/////////////////////////////////////////////
	on roll_automat_MatSmoothTool open do
	(
		getapref = execute ( getINISetting CurrentINI "User Settings" "Material Mode State" )
			if getapref != OK do 
			(
				but_MaterialMode.checked = getapref
				
				but_SmoothMode.checked = ( if getapref then false else true )
			
			)
			
			getapref = execute ( getINISetting CurrentINI "User Settings" "Replace Smoothing Mode" )
			if getapref != OK do Check_ReplaceSmooth.checked = getapref
		
	)
	
	on roll_automat_MatSmoothTool close do
	(
		setapref =  setINISetting CurrentINI "User Settings" "Material Mode State" (but_MaterialMode.checked as string )
		
		setapref =  setINISetting CurrentINI "User Settings" "Replace Smoothing Mode" (Check_ReplaceSmooth.checked as string )
	)
	
	
	on roll_automat_MatSmoothTool moved ScreenPos do
		(
			setapref =  setINISetting CurrentINI "MatSmooth" "RolloutPosition" (ScreenPos as string )
		)
		
	
	on but_MaterialMode changed state do
	(
		but_SmoothMode.checked = ( if state then false else true )
	)
	
	--
	
	on but_SmoothMode changed state do
	(
		but_MaterialMode.checked = ( if state then false else true )
	)
	
	--/////////////////////////////////////////////
		
		function processMatSmoothButton selection =
		(
			ReplaceSmoothBool=(if Check_ReplaceSmooth.state==true then false else true)
			SetMatOrSmooth=(if but_MaterialMode.checked then "mat" else "smooth")
			SmoothingGroupNumber=( if MatSmoothNumSelector<=1 then 1 else 2 ^ (MatSmoothNumSelector-1) )
				
			objArray = for piggy in selection where classof(piggy.baseobject) == Editable_Poly or classof(piggy) == PolyMeshObject collect piggy
			
			if GetOrSet=="set" then
			(
				if  SetMatOrSmooth=="mat" then
				(
					--just switch to faces because assigning mat IDs to a single element doesn't work properly (all faces get new ID)
					switchedto4 = false
					if subobjectlevel==5 do 
					(
						switchedto4 = true
						subobjectlevel=4
					)
					setMaterialIDForSelection (objArray) bool_DoRandom MatSmoothNumSelector
					if switchedto4 do subobjectlevel=5
				)
					else setSmoothIDForSelection (objArray)
				
			)
				
			else 
			(
				if  SetMatOrSmooth=="mat" then getMaterialIDForSelection (objArray) bool_DoRandom MatSmoothNumSelector
					else getSmoothIDForSelection (objArray)
			)	
			redrawViews()
		)
		
		function checkForModKeys =
		(
			if keyboard.controlPressed and keyboard.shiftPressed then MatSmoothNumSelector += 20
			else if keyboard.controlPressed do MatSmoothNumSelector += 10
		)
		--Buttons
		--/////////////////////////////////////////////
		--------------------------------------------------------
		on but_MaterialID_01 pressed do
		(
			MatSmoothNumSelector = 1
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_01 rightclick do
		(
			MatSmoothNumSelector=1
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_02 pressed do
		(
			MatSmoothNumSelector=2
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_02 rightclick do
		(
			MatSmoothNumSelector=2
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_03 pressed do
		(
			MatSmoothNumSelector=3
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_03 rightclick do
		(
			MatSmoothNumSelector=3
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_04 pressed do
		(
			MatSmoothNumSelector=4
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_04 rightclick do
		(
			MatSmoothNumSelector=4
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_05 pressed do
		(
			MatSmoothNumSelector=5
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_05 rightclick do
		(
			MatSmoothNumSelector=5
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_06 pressed do
		(
			MatSmoothNumSelector=6
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_06 rightclick do
		(
			MatSmoothNumSelector=6
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_07 pressed do
		(
			MatSmoothNumSelector=7
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_07 rightclick do
		(
			MatSmoothNumSelector=7
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_08 pressed do
		(
			MatSmoothNumSelector=8
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_08 rightclick do
		(
			MatSmoothNumSelector=8
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_09 pressed do
		(
			MatSmoothNumSelector=9
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_09 rightclick do
		(
			MatSmoothNumSelector=9
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MaterialID_10 pressed do
		(
			MatSmoothNumSelector=10
			checkForModKeys()
			GetOrSet="set"
			processMatSmoothButton (selection)
		) 
		on but_MaterialID_10 rightclick do
		(
			MatSmoothNumSelector=10
			checkForModKeys()
			GetOrSet="get"
			processMatSmoothButton (selection)
		) 
		
		--------------
		on but_MatSmoothlID_Random pressed do
		(
			bool_DoRandom=true
			MatSmoothNumSelector=(random 1 10)
			SetMatOrSmooth= "mat"
			--SetMatOrSmooth=(if but_MaterialMode.checked then "mat" else "smooth")
			GetOrSet= "set"
			objArray = for piggy in selection where classof(piggy.baseobject) == Editable_Poly collect piggy
			
		
			if  SetMatOrSmooth=="mat" then setMaterialIDForSelection (objArray) bool_DoRandom MatSmoothNumSelector
				else setSmoothIDForSelection (objArray)
			bool_DoRandom=false
			redrawViews()
			
		) 
		on but_MatSmoothlID_Random rightclick do
		(
			bool_DoRandom=true
			MatSmoothNumSelector=(random 1 10)
			SetMatOrSmooth= "mat"
			--SetMatOrSmooth=(if but_MaterialMode.checked then "mat" else "smooth")
			GetOrSet= "set"
			objArray = for piggy in selection where classof(piggy.baseobject) == Editable_Poly collect piggy
			
			
			if  SetMatOrSmooth=="mat" then getMaterialIDForSelection (objArray) bool_DoRandom MatSmoothNumSelector
				else getSmoothIDForSelection (objArray)
			redrawViews()	
		) 
		
		
		
		on but_ClearMats pressed do
		(	
			ObjArray = for piggy in selection where true == isValidNode(piggy) collect piggy
			undo on
			(
				for EachObject in ObjArray do EachObject.material = undefined
			)
		)
		--------------
		on but_ClearSmoothing pressed do
		(	
			MatSmoothNumSelector=0
			GetOrSet="set"
			SmoothingGroupNumber=0
			ReplaceSmoothBool=false
			objArray = for piggy in selection where classof(piggy.baseobject) == Editable_Poly collect piggy
			setSmoothIDForSelection (objArray)
			redrawViews()
		)
		
		on but_ApplyTestMat pressed do
		(
			if selection.count > 0 then 
			(
				--first check for material in scene
				scenefound = for each in scenematerials where each.name == "Automat_TestMaterial" collect each
				meditfound = for each in meditMaterials where each.name == "Automat_TestMaterial" collect each
					
				if not keyboard.altpressed then
				(

					--delete existing Automat material
					if true == querybox\
						"Delete Any Existing Versions of Automat_TestMaterial First?\n\nThis requires a scene hold."\
					do
					(

						for ping in Objects do --remove mat from Objects in Scene
						(
							if ping.material != undefined and ping.material.name == "Automat_TestMaterial" do ping.material = undefined --remove mat if found on object
							
						)
						for pong = 1 to 24 do --remove mat from material editor
						(
							if meditmaterials[pong].name == "Automat_TestMaterial" do meditmaterials[pong] = standard() --replace mat with new shader
							
						)
						
						max hold
						
						getUserInput_MakeMaterial() --put up a rollout with input for tiling info
						
					)--end do
					

				)
				else
				(
					if scenefound.count > 0 then
					(
						fallbackmaterial = scenefound[1] --name match found in scene, let's use that.
						$.material = fallbackmaterial
					)
					else if meditfound.count > 0 then
					(
						fallbackmaterial = meditfound[1]
						$.material = fallbackmaterial
					)
						else --make a new Automat_TestMaterial
					(
						getUserInput_MakeMaterial() --put up a rollout with input for tiling info
					) 
					
				)--end else
			
						
			)--end if then selection
			else
			(
				messagebox "Please pick an object first."
			)
					
		)--end button
			
	)
	
	
	--FUNCTIONS
	--/////////////////////////////////////////////
	--------------------------------------------------------
-- 	setMaterialIDForSelection --MOVED to BurkeLib
-- 	getMaterialIDForSelection
		
	--------------------------------------------------------
	function setSmoothIDForSelection objArray =
	(
		max modify mode
	
		with redraw off
		for i = 1 to objArray.count do
			
		(
			if classof(ObjArray[i].baseobject) == Editable_Poly do
			(
				
				if subobjectlevel==4  do
				(	
					tempForTesting=polyop.getFaceSelection  $
					polyop.setFaceSmoothGroup $ tempForTesting (SmoothingGroupNumber) add: ReplaceSmoothBool
					--format "MatSmoothNumSelector %" MatSmoothNumSelector
					--print ReplaceSmoothBool
				)
				
				
				if subobjectlevel==0 do
				(
					wiggly=polyop.getnumfaces objArray[i].baseobject
					for j=1 to wiggly do polyop.setFaceSmoothGroup objArray[i].baseobject j SmoothingGroupNumber add: ReplaceSmoothBool
					--print ReplaceSmoothBool
				)
			
			)
			
			
		)	
	)
	
	--------------------------------------------------------
	function getSmoothIDForSelection objArray =
	(	
		for i = 1 to objArray.count do
			
		(
			if classof(ObjArray[i].baseobject) == Editable_Poly do
			(
				
				AllTheFaces=polyop.getnumfaces objArray[i].baseobject --how many faces in current object
				local FaceSelectionArray=#{} --empty the bit arrray gonna use to collect faces that match the ID am looking for
			
				for CurrentCounter=1 to AllTheFaces do
				(
					if polyOp.getFaceSmoothGroup objArray[i].baseobject CurrentCounter == SmoothingGroupNumber do
					(
						append FaceSelectionArray CurrentCounter
					)
				)
				polyop.setFaceSelection objArray[i].baseobject FaceSelectionArray
			)
		)
	)
	
	--------------------------------------------------------
	if true != automat_OpenScriptAsRollout then
	(
		try (closeRolloutfloater automat_MatSmoothTool ) catch ()
		automat_MatSmoothTool = newRolloutFloater "Mat/Smooth Tool" 170 102 RolloutPosition.x RolloutPosition.y
		addRollout roll_automat_MatSmoothTool automat_MatSmoothTool
		automat_MatSmoothTool.size.y = ( roll_automat_MatSmoothTool.height + 30 )
	)
	else()
)