
--	ModelDiag.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.6
--	CREATED:	02-19-2011 --3dsmax 2011

--04-21-2014	Fixed issue with kerning, improved measurement tags for really small scenes
--04-21-2014	Now uses display units for labeling measurements, user-configured wirecolors for objects, added scale multiplier to set tag object size
--03-22-2014   Added warning about Highlight w/Mats, Mat Ids
--5-10-11	Added Kill Empty Objects Feature
--5-5-11	Added Find DingleVert function. 
--4-21-11 Improved ability to find and select n-gons apart from tris and quads.
--4-20-11 Added ability to identify and select faces with more than four verts separately from tris

(
	global automat_ModelDiag
	
	local CurrentINI = GetDir #plugcfg + "\\automat_ModelDiag.ini"
	
	--------------------------------------------------------
	if true != BurkeLibLoaded do include "$userScripts/BurkeStudios/lib/BurkeLib.ms"
	
	--------------------------------------------------------
	--Check for Quads
	--------------------------------------------------------
	local QuadReportText=#("Quads", "Non-Quads","-")
	local LookForThis = "nonquads"
	local DiagObjArray = undefined
	local MisDiagObjArray = undefined
	--
	local ArrayOfNamesQuads=#()
	local ArrayOfNamesNonQuads=#()
	
	
	--Open Icons w/ proper gamma
	openBurkeIconsWithProperGamma()
	
	
	
	--function names
	--------------------------------------------------------
	local findNonQuad
	
	local checkObjectNames
	
	--------------------------------------------------------
	--Tag Objects
	--------------------------------------------------------
	local MeasureArray=#()
	local MeasureTextArray=#()
	--
	global LastMeasureObject
	--function names
	local deleteLastMeasureObject
	
	local getMeasureForSelection
	local createNameTags
	local deleteNameTags
	local findDodgyVerts
	
	local wirecolor
	local scalemult --multiplier, from spinner, to scale up or down the various tag constructions
	
	--INI settings
	--------------------------------------------------------
	local UseDiagMat = true --default
	--------------------------------------------------------
	
	--UI
	--/////////////////////////////////////////////
	--------------------------------------------------------
	rollout Roll_automat_ModelDiag "Diagnostic Tools"
	(
		
		group "Check for quads:"
		(
			button but_CheckObject "Check Selection" offset:[3,0] tooltip:"Check one or many Editable Poly objects for quad polygons"
			
			button but_IconTest1 width:24 height:24 border:false enabled:true across:2 offset:[-24,-2] \
				images:#(Icons,"$icons\BURKE-Icons2_24a.bmp", IconsInFile,9,9,9,9, true ) tooltip:"Select Quads/Objects built of all quads"
			editText text_QuadReport1 text:QuadReportText[1] readonly:true width:116 offset:[-46,0] border:false
			
			button but_IconTest2 width:24 height:24 border:false enabled:true across:2 offset:[-24,-2] \
				images:#(Icons,"$icons\BURKE-Icons2_24a.bmp", IconsInFile,4,4,4,4, true ) tooltip:"Select Triangles/Objects containing triangles"
			editText text_QuadReport2 text:QuadReportText[2] readonly:true width:116 offset:[-46,0] border:false
			
			button but_IconTest3 width:24 height:24 border:false enabled:true across:2 offset:[-24,-2] \
				images:#(Icons,"$icons\BURKE-Icons2_24a.bmp", IconsInFile,5,5,5,5, true ) tooltip:"Selects n-gons\n(when a single E-poly object is selected)"
			editText text_QuadReport3 text:QuadReportText[3] readonly:true width:116 offset:[-46,0] border:false
			
			checkbox check_UseDiagMat "Hilight w/ Mats, Mat IDs" checked:false
		)
		--------------
		group "Find Dodgy Verts:"
		(
			button but_FindDingles "Find and Select" offset:[3,0] tooltip:"Check selected Editable Poly object for verts that have less than three edges. Often these verts are problematic."
			button but_IconTest4 width:24 height:24 border:false enabled:true across:2 offset:[-24,-2] \
				images:#(Icons,"$icons\BURKE-Icons2_24a.bmp", IconsInFile,8,8,8,8, true )
			editText text_DingleReport text:"-" readonly:true width:116 offset:[-46,0] border:false
			
		)
		--------------
		group "Verify Unique Names:"
		(
			button but_VerifyUnique "Check Selection" tooltip: "Checks selection of objects to ensure that all object names are unique."
		)
		--------------
		group "Empty E-Poly Objects:"
		(
			button but_KillEmpties "Find and Delete" tooltip:"Finds any EPoly objects in scene that contain no verts and deletes them."
			edittext text_KillEmptiesReport "" width:140 offset:[-6,0] readonly:true
		)
		--------------
		group "Tag Objects:"
		(
			colorpicker col_namecolor "" color:[18,17,29] width:16 across:4 offset:[-10,0] \
			tooltip:"Wirecolor for name tags"
			
			button but_CreateNameTags "Names"  width:38 offset:[-17,0]  \
			tooltip:"Left-click: Creates a name tag object for each object in selection.
\nRight-click: Deletes name tags from selection."
			
			colorpicker col_measurecolor "" color:[13,12,10] width:16 offset:[-20,0] \
			tooltip:"Wirecolor for measurement tags"
			button but_MeasureTool "Measurements" width:75 offset:[-8,0]\
			tooltip:"Left-click: Creates a measure object for current selection.
\nRight-click: Deletes last created measure object."
			
			spinner spinScaleMultiplier "Tag Scale Multiplier" range:[.01,100,1] fieldWidth: 34 \
			tooltip: "Make the tag objects bigger or smaller to fit your scene scale."
			
			
		)
		
		--INI related
		--------------------------------------------------------
		on Roll_automat_ModelDiag open do
		(
			getapref = execute ( getINISetting CurrentINI "Diag Settings" "UseDiagMat" )
			if getapref != OK do check_UseDiagMat.checked = ( UseDiagMat = getapref )
			
			getapref = execute ( getINISetting CurrentINI "Diag Settings" "WireColor Names" )
			if getapref != OK do col_namecolor.color = ( WireColor = getapref )
			
			getapref = execute ( getINISetting CurrentINI "Diag Settings" "WireColor Measurements" )
			if getapref != OK do col_measurecolor.color = ( WireColor = getapref )
			
			getapref = execute ( getINISetting CurrentINI "Diag Settings" "Scale Multiplier" )
			if getapref != OK do spinScaleMultiplier.value = ( scalemult = getapref )
		)
		
		on check_UseDiagMat changed state do
		(
			
			if true == state then if Querybox ( "This feature highlights problem faces by changing the material and material ID assignments of the model; if those are precious to you don't use this option.\n\nUse this feature?" )
			then
			(
				UseDiagMat = state
				setapref =  setINISetting CurrentINI "Diag Settings" "UseDiagMat" (UseDiagMat as string )
			)
			else
			(
				check_UseDiagMat.checked = false --uncheck the box
			)
			
		)
		
		on col_namecolor changed NewColor do
		(
			setapref =  setINISetting CurrentINI "Diag Settings" "WireColor Names" ( NewColor as string )
		)
		
		on col_measurecolor changed NewColor do
		(
			setapref =  setINISetting CurrentINI "Diag Settings" "WireColor Measurements" ( NewColor as string )
		)
		
		on spinScaleMultiplier changed NewValue do
		(
			setapref =  setINISetting CurrentINI "Diag Settings" "Scale Multiplier" ( NewValue as string )
		)
		
		--BUTTONS
		--/////////////////////////////////////////////
		--------------------------------------------------------
		on but_CheckObject pressed do
		(
			FreshCheck = true --don't select objects on a first check
			DiagObjArray = undefined
			MisDiagObjArray = undefined
			UseDiagMat = check_UseDiagMat.checked
			LookForThis = "nonquads"
			findNonQuad selection LookForThis UseDiagMat	--Function call
			text_QuadReport1.text = QuadReportText[1] --update text
			text_QuadReport2.text = QuadReportText[2] --update text
			text_QuadReport3.text = QuadReportText[3] --update text
			FreshCheck = false
			
		)

		on but_IconTest1 pressed do
		(
			FreshCheck = false
			LookForThis = "quads"
			findNonQuad selection LookForThis UseDiagMat
			text_QuadReport1.text = QuadReportText[1] --update text
			text_QuadReport2.text = QuadReportText[2] --update text
			text_QuadReport3.text = QuadReportText[3] --update text
		)
			
		on but_IconTest2 pressed do
		(
			FreshCheck = false
			LookForThis = "nonquads"
			findNonQuad selection LookForThis UseDiagMat
			text_QuadReport1.text = QuadReportText[1] --update text
			text_QuadReport2.text = QuadReportText[2] --update text
			text_QuadReport3.text = QuadReportText[3] --update text
		)
		
		on but_IconTest3 pressed do
		(
			FreshCheck = false
			LookForThis = "morefour"
			findNonQuad selection LookForThis UseDiagMat
			text_QuadReport1.text = QuadReportText[1] --update text
			text_QuadReport2.text = QuadReportText[2] --update text
			text_QuadReport3.text = QuadReportText[3] --update text
		)
		
		on but_FindDingles pressed do
		(
			findDodgyVerts selection
		)
		--
		on but_IconTest4 pressed do
		(
			findDodgyVerts selection
		)
		
		--------------------------------------------------------
		on but_KillEmpties pressed do
		(
			ObjArray = for each in Objects where classof (each.baseobject) == Editable_Poly collect each

			if ObjArray.count != 0 then
			(
				local EmptyObjectArray=#()
				--
				local gv = polyop.GetNumVerts
				local gf = polyop.getNumFaces
				
				for each in ObjArray do
				(
					theobject = each.baseobject
					if 0 == gv theobject and 0 == gf theobject do append EmptyObjectArray each	
				)
			
				delcount = EmptyObjectArray.count
				
				if 0 == delcount then text_KillEmptiesReport.text = "OK! None found in scene."
					else if 1 == delcount then text_KillEmptiesReport.text = EmptyObjectArray[1].name + " Removed."
						else text_KillEmptiesReport.text = delcount as string + " Objects Removed"
				--
				delete EmptyObjectArray
			)
			else text_KillEmptiesReport.text = "No EPoly Objects in Scene"
		)
		
		on but_VerifyUnique pressed do
		(
			checkObjectNames selection
		)
		
		--------------------------------------------------------
		--Buttons Tag Tool
		--------------------------------------------------------
		on but_MeasureTool pressed do
		(

			if selection.count > 0 then
			(
				wirecolor = col_measurecolor.color
				scalemult =  spinScaleMultiplier.value
				getMeasureForSelection Selection --FUNCTION
			)
			else messagebox "Please select one or more objects to measure."
		)
		
		
		on but_MeasureTool rightclick do
		(
			deleteLastMeasureObject()	
		)
		--------------
		on but_CreateNameTags pressed do
		(
			wirecolor = col_namecolor.color
			scalemult = spinScaleMultiplier.value
			createNameTags selection
		)
		on but_CreateNameTags rightclick do
		(
			deleteNameTags selection
		)
	
		
	)--rollout
	
	--FUNCTIONS
	--/////////////////////////////////////////////
	--------------------------------------------------------
	function findNonQuad selection LookForThis UseDiagMat =
	(
		if 1 == selection.count and classof(selection[1].baseobject ) == Editable_Poly then
		(
			
			
			
			--SINGLE OBJECT MODE
			local FaceSelectionArray = #{}
			local MoreFourSelectionArray = #{}
			local NonQuadFacesSum = 0
			local QuadFacesSum = 0
			local MoreThanFour = 0

			local CurrentObj = selection[1]
			local BaseObj = $.baseobject
			local FaceCount = polyop.getNumFaces BaseObj
			
			--get user's current SubObject Level
			local currentsolevel = subobjectlevel
			
			--------------
			--ASSIGN TEMP MATERIAL - later I'll add a restore function
			if true == UseDiagMat do
			(
				if undefined == automat_Materials do automat_Materials = loadTempMaterialLibrary "Automat_Materials.mat" --LOAD LIBRARY IF NEEDED
					CurrentObj.material=automat_materials[13]
			)
			--------------
			
			for loop = 1 to FaceCount do 
			(
				local FaceVertCount = GetFaceDeg BaseObj loop
				if FaceVertCount != 4 then --Non-quads here
				(
					if 3 == FaceVertCount then --triangles
					(
						if "nonquads" == LookForThis do FaceSelectionArray[loop] = true
						if true == UseDiagMat do SetThisFaceMatID CurrentObj loop 2
						NonQuadFacesSum += 1 --NON-QUAD
					)
					--------------
					else --we have ngons
					(
						MoreThanFour += 1 --count bigger than quads
						MoreFourSelectionArray[loop] = true
						if "morefour" == LookForThis do FaceSelectionArray[loop] = true
						if true == UseDiagMat do SetThisFaceMatID CurrentObj loop 3
					)
					--------------
				)
				else --We have quads
				(
					QuadFacesSum += 1 --QUAD
					if ("quads" == LookForThis) do FaceSelectionArray[loop] = true
					if true == UseDiagMat do SetThisFaceMatID CurrentObj loop 1
				)
			)	
			
			
			if 0  != NonQuadFacesSum or 0 != MoreThanFour or ("nonquad" == LookForThis) then --there are some non-quad faces, select them.
			(
				polyop.setFaceSelection CurrentObj  FaceSelectionArray
				max modify mode
			
				modPanel.setCurrentObject CurrentObj
				subobjectlevel = 4
				
				QuadReportText[1] = if 0 == QuadFacesSum then "-"
					else if 1 == QuadFacesSum then "1 Quad"
						else QuadFacesSum as string + " Quads"
				QuadReportText[2] = if 0 == NonQuadFacesSum then "-"
					else if 1 == NonQuadFacesSum then "1 Triangle"
						else NonQuadFacesSum as string + " Triangles"
				--
				if 1 == MoreThanFour then QuadReportText[3] = "1 N-Gon!"
					else if MoreThanFour >0 then QuadReportText[3] = MoreThanFour as string + " N-Gons!"
						else QuadReportText[3] = "-" --+ CurrentObj.name
					
				--restore user subobject mode if user is looking for ngons and there aren't any
				if 0 == MoreThanFour and ("morefour" == LookForThis) do subobjectlevel = currentsolevel
				
-- 				--restore user subobject mode if user is looking for tris and there aren't any
-- 				if 0 == NonQuadFacesSum and ("nonquad" == LookForThis) do subobjectlevel = 0
-- 					
-- 				--restore user subobject mode if user is looking for quads and there aren't any
-- 				if 0 == QuadFacesSum and ("quad" == LookForThis) do subobjectlevel = 0
				
			)
	-- 		if 0 == NonQuadFacesSum and DiagObjArray != undefined and ArrayOfNamesNonQuads.count > 0 and selection.count ==1 then
	-- 		(
	-- 			select ArrayOfNamesNonQuads
	-- 		)
			else
			(
				--THIS OBJECT is ALL QUADS
				subobjectlevel = 0
				QuadReportText[1] = "OK! All Quads"
				QuadReportText[2] = "-"
				QuadReportText[3] = CurrentObj.name
				
			)
			
		)
		else if 0 < selection.count do
		(
			--MULTI-OBJECT MODE
			local ArrayOfNamesQuads=#()
			local ArrayOfNamesNonQuads=#()
			local ArrayOfNamesUnchecked=#()
			--------------
			
					
			if undefined == DiagObjArray do DiagObjArray = for piggy in selection where classof(piggy.baseobject ) == Editable_Poly collect piggy
			if undefined == MisDiagObjArray do MisDiagObjArray = for piggy in selection where classof(piggy.baseobject ) != Editable_Poly collect piggy
			
			
				--ASSIGN TEMP MATERIAL - later I'll add a restore function
			if true == UseDiagMat do
			(
				if undefined == automat_Materials do automat_Materials = loadTempMaterialLibrary "Automat_Materials.mat" --LOAD LIBRARY IF NEEDED
				if true == UseDiagMat do	for winky in DiagObjArray do winky.material = automat_materials[13]
			)
			--------------

			
			for wiggler in DiagObjArray do
			(
				local FaceSelectionArray = #{}

				local BaseObj = wiggler.baseobject
				
				local FaceCount = polyop.getNumFaces BaseObj
				loop = 1
				while 4 == GetFaceDeg BaseObj loop and loop < FaceCount do loop += 1
				if loop < FaceCount then append ArrayOfNamesNonQuads wiggler
					else append ArrayOfNamesQuads wiggler
			)
			
			--------------
			if true == UseDiagMat do
			(
					setMaterialIDForSelection ArrayOfNamesNonQuads false 2
					--MatSmoothNumSelector = 1
					setMaterialIDForSelection ArrayOfNamesQuads false 1
			)
			--------------
			
				if "nonquads" == LookForThis then
				(
					select ArrayOfNamesNonQuads
				)
				else if "quads" == LookForThis then 
				(
					select ArrayOfNamesQuads
				)
					else select  MisDiagObjArray
			
			
			
			QuadReportText[1] = if 0 == ArrayOfNamesQuads.count then "-"
				else if 1 == ArrayOfNamesQuads.count then "1 Quad Object"
					else ArrayOfNamesQuads.count as string +  " Quad Objects"
			QuadReportText[2] = if 0 == ArrayOfNamesNonQuads.count then "-"
				else if 1 == ArrayOfNamesNonQuads.count then "1 Non-quad Object"
				else ArrayOfNamesNonQuads.count as string +  " Non-quad Objects"
			--------------
			QuadReportText[3] = if  0 == MisDiagObjArray.count then "-"
				else if 1 == MisDiagObjArray.count then "1 Unchecked Object"
				else MisDiagObjArray.count as string +  " Unchecked Objects"
		)
		--RETURN AND DO NOTHING
	)
	
	--------------------------------------------------------
	function findDodgyVerts selection =
	(
		ObjArray = for each in selection where classof (each.baseobject) == Editable_Poly collect each
		
		if ObjArray.count != 1 then messagebox "Please select a single editable poly object."
			else
		(
			theobject = ObjArray[1].baseobject
			
			local VertCount = polyop.GetNumVerts theobject
			local VertSelectionArray = #{}
			local VertSelectCount = 0
			
			for loop = 1 to VertCount do
			(
				hit = theobject.GetVertexEdgeCount loop
				if hit <= 2 then
				(
					testresult = true
					VertSelectCount += 1
				)
				else testresult = false
				
				VertSelectionArray[loop] = testresult
			)
			
			theobject.SetSelection #Vertex VertSelectionArray
			if VertSelectCount > 0 then
			(
				if 1 == VertSelectCount then Roll_automat_ModelDiag.text_DingleReport.text = "1 Dodgy Vert"
					Roll_automat_ModelDiag.text_DingleReport.text = VertSelectCount as string + " Dodgy Verts"
				subObjectLevel = 1
			)
			else Roll_automat_ModelDiag.text_DingleReport.text = "OK!" + " " + ObjArray[1].name	
		)
		
	)
	
	--------------------------------------------------------
	
	-- 9-1-11 first pass only checks for object name uniqueness. Later I'll probably add checks for names that don't export correctly because of special characters. I may craft into a general export prep tool. Who knows?
	
	fn checkObjectNames selection =
	(
		objectcount = selection.count --get count of selected objects
		
		if objectcount == 0 then messagebox "No objects selected"
			else
			(
				duplicatenamesarray=#()
				tempobjnamearray=#() --gonna append names here if they are unique
				
				for each in selection do
				(
					beforecount = tempobjnamearray.count
					appendifunique tempobjnamearray each.name --adds name of each object in scene to array
					if tempobjnamearray.count == beforecount then append duplicatenamesarray each
				)
				
				differencecount = duplicatenamesarray.count
				
				if 0 == differencecount then messagebox "OK! All names unique!"
					else
				(
					select duplicatenamesarray
					messagebox ( "WARNING!!!! \n\nProblem names found: " + ( differencecount as string ) + "\n\The selected objects do not have unique names.")
				)
			)
	)
	
	
	--------------------------------------------------------
	--FUNCTIONS Tag Objects
	--------------------------------------------------------
	--/////////////////////////////////////////////

	
	
	--------------------------------------------------------
	function getMeasureForSelection Selection =
	(
		buff=0.2 --SPACER FOR OBJECTS
		buff2=0.5 --SPACER FOR TEXT
		MeasureArray=#() --EMPTY
		
		--
		for i = 1 to 3 do MeasureArray[i]=Selection.max[i]-Selection.min[i] --STORE X,Y,Z MEASURES
		
		---
		
		
		
		undo off --CUZ THIS IS UGLY
		(
			local switchedmodes = false
			if (getCommandPanelTaskMode() == #modify) do
			(
				max create mode -- SPEEDS THINGS UP
				switchedmodes = true
			)
			with redraw off
			(
				
				sm = scalemult
				local xbuff = ( if sm < .25 then ( sm * 4 ) else 1 )
			
				--BUILD Z stick
				measureCyl=convertToPoly(cylinder  name: "MeasureStick_000" height:MeasureArray[3] radius: ( sm * 1 ) sides:6 heightsegs:4)
				measureCyl.position=[ Selection.max[1] + ( abs Selection.max[1] ) * buff , Selection.max[2],Selection.min[3] ]
				measurelabel = text text: ( units.formatValue MeasureArray[3] ) size:( sm * 10 ) alignment:1 kerning: 0 \
					render_renderable:false render_displayRenderMesh:false
				measurelabel.pivot = measurelabel.min
				measurelabel.position = [ Selection.max[1] + ( abs Selection.max[1] ) * buff + ( sm * 4 )  , Selection.max[2], Selection.max[3] - (sm * 7 ) ]
				rotate measurelabel (quat 90 x_axis) 
				polyop.attach MeasureCyl measurelabel
				
				
				--BUILD Y stick
				measureCylY=convertToPoly(cylinder  height:MeasureArray[2] radius: ( sm * 1 ) sides:6 heightsegs:4)
				measureCylY.position=[ Selection.max[1] + ( abs Selection.max[1] ) * buff, Selection.min[2],Selection.center[3] ]
				rotate measureCylY (quat -90 x_axis )
				measurelabel=text text: ( units.formatValue MeasureArray[2] ) size: ( sm * 10 ) alignment:1 kerning:0 \
					render_renderable:false render_displayRenderMesh:false
				measurelabel.pivot= measurelabel.min
				measurelabel.position=[ Selection.max[1] + ( abs Selection.max[1] ) * buff, Selection.min[2], Selection.center[3] + ( sm * 4 ) ]
				rotate measurelabel (quat 90 x_axis )
				rotate measurelabel (quat 90 z_axis )
				
				polyop.attach MeasureCyl measureCylY
				polyop.attach MeasureCyl measurelabel
				
				--BUILD X stick
				quarterheight=(Selection.min[3]+(MeasureArray[3]/4))
				measureCylZ=convertToPoly(cylinder  height:MeasureArray[1] radius:( sm * 1 ) sides:6 heightsegs:4)
				
				
					
				measureCylZ.position=[ Selection.min[1], Selection.min[2] - ( abs Selection.min[2] ) * buff * xbuff , quarterheight ]
				rotate measureCylZ (quat 90 y_axis )
				measurelabel=text text: ( units.formatValue MeasureArray[1] ) size: ( sm * 10 ) alignment:1 kerning: 0 \
					render_renderable:false render_displayRenderMesh:false
				measurelabel.pivot= measurelabel.min
				measurelabel.position=[ Selection.min[1], Selection.min[2] - ( abs Selection.min[2] ) * buff * xbuff , quarterheight + ( 4 * sm) ]
				rotate measurelabel (quat 90 x_axis )
				rotate measurelabel (quat 0 z_axis )
					
				polyop.attach MeasureCyl measureCylZ
				polyop.attach MeasureCyl measurelabel
				
				
				initialmeasure= (getNodeByName "MeasureStick_000")
				
				initialmeasure.wirecolor = wirecolor
				initialmeasure.pivot = selection.center
				
				-- ADD TO LAYER
				layermanager.newlayerfromname "-- Measurement Tools"
				WorkingLayer = layermanager.getLayerFromName "-- Measurement Tools"
				WorkingLayer.addnode initialmeasure
				
				maxOps.CloneNodes  initialmeasure newNodes:&LastMeasureObject 	--MAKE MAX FIND UNIQUE NAME
				delete initialmeasure --SAVE 00 NAME FOR ALL BUILDS
			)--END CONSTRUCTION OF MEASURE OBJECT
			
			if true == switchedmodes do max modify mode --restore modify mode
		)
		
			
		redrawViews()	
	)
	
	function deleteLastMeasureObject =
	(
		try( delete LastMeasureObject)catch()
		
	)
	
	--------------------------------------------------------
	function createNameTags selection = 
	(
		ObjArray = for piggy in selection where superclassof(piggy) == GeometryClass or superclassof(piggy)==shape collect piggy
		
		if ObjArray.count > 0 then 
		(
			if (getCommandPanelTaskMode() == #modify) do
			(
				max create mode -- SPEEDS THINGS UP
				switchedmodes = true
			)
			
			local TempCollectionOfLabels=#()
			
			undo off
			(
				sm = scalemult
				
				for wiggly in ObjArray do
				(
					LabelObj =  text  text:wiggly.name size: ( sm * 10 ) alignment:1 kerning: 0 \
						render_renderable:false render_displayRenderMesh:false  --FIXED SIZE
					LabelObj.pos = [ (wiggly.center.x), (wiggly.center.y),  (wiggly.min.z) ] --FIXED OFFSET FOR NOW
					move LabelObj [0,0, ( sm * -10 ) ]
					ep = Edit_Poly()
					addmodifier labelObj ep
					
					--LabelObj.pivot= LabelObj.min
					rotate LabelObj (quat 90 x_axis) 
					
					LabelObj.name = uniquename ("automat_NameTag_" + wiggly.name + "_" ) 
					LabelObj.parent = wiggly
					LabelObj.wirecolor = wirecolor
					LabelObj.pivot = wiggly.pivot
					
					append TempCollectionOfLabels LabelObj
				)
				
				--format "winky %\n\n" TempCollectionOfLabels
				clearSelection()
				for winky in TempCollectionOfLabels do selectmore winky
			)
			
			if true == switchedmodes do max modify mode --restore modify mode
		)
		else messagebox "No objects selected."
		--------------
		redrawViews()
	)
			
	--------------------------------------------------------
	function deleteNameTags selection = 
	(
		ObjArray = for piggy in selection where ( matchPattern piggy.name pattern:"automat_NameTag_*" == true ) collect piggy
		
		if ObjArray.count > 0 do 
		(
			for wiggly in ObjArray do
				(
					delete wiggly
				)
			
		)
		--redrawViews()
	)
	--------------------------------------------------------
	
	
	if true != automat_OpenScriptAsRollout then
	(
		try (closeRolloutfloater automat_ModelDiag ) catch ()
		automat_ModelDiag = newRolloutFloater "Model Diagnostics" 170 102
		addRollout roll_automat_ModelDiag automat_ModelDiag
		automat_ModelDiag.size.y = ( roll_automat_ModelDiag.height + 30 )
	)
	else()
)