
--	MeshZipper.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.1
--	CREATED:	02-19-2011 --3dsmax 2011
--	MODIFIED:	09-12-2013--3dsmax 2014
--					

--Change log
--04-01-2011	Improved logic for when user selection is very different from auto-selection. Reruns vert function but uses existing selection instead of auto-selection. added auto-hold setting and prefs.
--02-21-2011	Added callback function
--09-12-2013	Amend ini file in order to save and load the Tolerance setting

(
	global automat_ModelZipper
	global roll_automat_ModelZipper
	
	local CurrentINI = GetDir #plugcfg + "\\automat_ModelZipper.ini"
	--
	callbacks.addScript #selectionSetChanged "roll_automat_ModelZipper.selectionChangeFunction()" id:#AutomatModelZipper
	
	
	struct struct_ModelZipper
	(
		autohold = true	
	)
	local ZipPrefs = struct_ModelZipper()
	
	if true != BurkeLibLoaded do include "$userScripts/BurkeStudios/lib/BurkeLib.ms"
	--------------------------------------------------------
	local GetVertAxis
	local VertStartPoint
	--------------
	local VertsByNumber
	local CountOfSelVerts
	--------------
	local SelectionOfVerts
	local VertCount
	local UseUserSelection = false

	--Function Names
	--------------
	local getVertsEdge
	local zipModelsTogether


	--UI
	--/////////////////////////////////////////////
	--------------------------------------------------------
	rollout roll_automat_ModelZipper "Model Zipper"
	(
		spinner spin_EdgeFudge "Tolerance:" range:[0,10,1] width:120 offset:[-3,0] across:1 tooltip:"Helps fine-tune the selection, if needed."

		label lab_GetVerts1 "Step 1:" align: #left offset:[-4,0] across:2
		button but_VertsEdge "Get Verts on Seam" width: 100 offset:[-10,-2] \
		tooltip:"First select Editable Poly object and then Click this button.\n
Model Zipper will attempt to select all of the verts along the largest open edge. For some models this will work perfectly.\n
Always double-check the automatic selection and change the selection as needed to ensure only the verts you want will be zipped."  
		
		label lab_GetVerts2 "Step 2:" align: #left offset:[-4,0] across:2
		pickButton but_ZipTo "Pick ZipTo Obj ..." width: 100 offset:[-13,-2] enabled:false tooltip:"This should be an instance of the first object.\n\nThe selected verts in current object will be automagically zipped to corresponding verts in Zip To Object." 
		checkbox check_AutoHold "Auto-hold before weld?" checked:true 
		label label_hold "(there is no undo)" offset:[-3,-4]
		
		--------------
		on but_VertsEdge pressed do
		(
			Tolerance = spin_EdgeFudge.value
			getVertsEdge selection Tolerance
		)
		
		--------------
		on but_ZipTo picked PickedObj do
		(
			if PickedObj != undefined and 1 == selection.count do
			(
				zipModelsTogether selection PickedObj
			)
		)
		
		--------------------------------------------------------
		function selectionChangeFunction =
		(
			if selection.count > 0 then
				(
					but_VertsEdge.enabled = true
				)
					else
				(
					but_VertsEdge.enabled = false
					but_ZipTo.enabled = false
				)
		)
		
		--------------
		on roll_automat_ModelZipper open do
		(
			getapref = execute ( getINISetting CurrentINI "Model Zipper" "Auto-hold enable" )
			if getapref != OK do ZipPrefs.autohold = check_AutoHold.checked = getapref
			--
			getapref = execute ( getINISetting CurrentINI "Model Zipper" "Tolerance" )
			if getapref != OK do spin_EdgeFudge.value = getapref
			--
			--callback
			selectionChangeFunction()
		)
		--------------
		on roll_automat_ModelZipper close do
		(
			callbacks.removeScripts #selectionSetChanged  id:#AutomatModelZipper
			--
			setapref =  setINISetting CurrentINI "Model Zipper" "Tolerance" (spin_EdgeFudge.value as string )
		)
		--------------
		on check_AutoHold changed state do
		(
			ZipPrefs.autohold = state
			setapref =  setINISetting CurrentINI "Model Zipper" "Auto-hold enable" (state as string )
		)
		
		
	)--END ROLLOUT

	--FUNCTIONS
	--/////////////////////////////////////////////
	--------------------------------------------------------
	function getVertsEdge selection Tolerance =
	(
		if 1 == selection.count and Editable_Poly == classof( selection[1] ) then
		(
			with redraw off
			(
				print "getVertsEdge Function"
				--
				subobjectlevel = 0
				CurrentObj = selection[1]
				CurrentObjMin = CurrentObj.min
				CurrentObjMax = CurrentObj.max
				
				if false == UseUserSelection then
				(
					print "Getting selection edges this time"
					selectionOfEdges = polyop.getOpenEdges CurrentObj
				
					SelectionOfVerts = polyop.getVertsUsingEdge CurrentObj selectionOfEdges
					max modify mode
					polyop.setVertSelection CurrentObj SelectionOfVerts
					subobjectlevel = 1
				)
				else SelectionOfVerts = polyop.getVertSelection CurrentObj --using current vert selection whatever that may be. User may have changed it from auto-select
				
				--------------
				--create an array of vert numbers for selection
				VertsByNumber = #()
				VertsByNumberPositions=#()
				AxisMinMaxValues=#()
				SumX = 0
				SumY = 0
				SumZ = 0
				VertCount = SelectionOfVerts.count
				--format "VertCount: %\n" VertCount
				
				for loop = 1 to VertCount do
				(
					if true == SelectionOfVerts[loop] do
					(
						--these are selected verts
						append VertsByNumber loop
						CurrentVert = VertPos CurrentObj loop
						append VertsByNumberPositions CurrentVert
						SumX += CurrentVert[1]
						SumY += CurrentVert[2]
						SumZ += CurrentVert[3]
					)
				)
			
				CountOfSelVerts = VertsByNumber.count
				
				if CountOfSelVerts > 0 then
				(
				
					roll_automat_ModelZipper.but_ZipTo.enabled = true
					
					--format "CountOfSelVerts: %\n" CountOfSelVerts
					AvgX = SumX / CountOfSelVerts
					AvgY = SumY / CountOfSelVerts
					AvgZ = SumZ / CountOfSelVerts
					--format "AvgX: %\n" AvgX
					--format "Avgy: %\n" Avgy
					--format "Avgz: %\n" Avgz
				
					--Find dif between average and min/max values for each axis
					--The axis I am looking for is going to be the one with the smallest difference
					AxisMinMaxValues[1] = abs ( AvgX - CurrentObjMin.x )
					AxisMinMaxValues[2] = abs ( AvgX - CurrentObjMax.x )
					AxisMinMaxValues[3] = abs ( AvgY - CurrentObjMin.y )
					AxisMinMaxValues[4] = abs ( AvgY - CurrentObjMax.y )
					AxisMinMaxValues[5] = abs ( AvgZ - CurrentObjMin.z )
					AxisMinMaxValues[6] = abs ( AvgZ - CurrentObjMax.z )
				
					AxisPick = amin AxisMinMaxValues
					--format "AxisPick: %\n" AxisPick
							
					case AxisPick of
					(
						( AxisMinMaxValues[1] ) : 
							(
								GetVertAxis = 1
								VertStartPoint = CurrentObjMin.x
								--print "x-min"
							)
						( AxisMinMaxValues[2] ) : 
							(
								GetVertAxis = 1
								VertStartPoint = CurrentObjMax.x
								--print "x-max"
							)
						( AxisMinMaxValues[3] ) : 
							(
								GetVertAxis = 2
								VertStartPoint = CurrentObjMin.y
								--print "y-min"
							)
						( AxisMinMaxValues[4] ) : 
							(
								GetVertAxis = 2
								VertStartPoint = CurrentObjMax.y
								--print "y-max"
							)
						( AxisMinMaxValues[5] ) :
							(
								GetVertAxis = 3
								VertStartPoint = CurrentObjMin.z
								--print "z-min"
							)
						( AxisMinMaxValues[6] ) : 
							(
								GetVertAxis = 3
								VertStartPoint = CurrentObjMax.z
								--print "z-max"
							)
					)
						
					ToleranceStep = ( distance CurrentObjMax CurrentObjMin ) / VertCount * 20  --rough measure
					--format "ToleranceStep: %\n" ToleranceStep
					ToleranceAmount = (Tolerance * ToleranceStep )
					--format "ToleranceAmount: %\n" ToleranceAmount
					local removedverts = false	
						
					for loop = 1 to CountOfSelVerts do
					(
						if  ( abs (VertsByNumberPositions[loop][GetVertAxis] - VertStartPoint )  > ToleranceAmount ) do SelectionOfVerts[ (VertsByNumber[loop] ) ] = false
						removedverts = true
					)
					
					if true == removedverts do 
					(
						VertsByNumber=#()
						for loop = 1 to VertCount do
						(
							if true == SelectionOfVerts[loop] do append VertsByNumber loop
						)
						CountOfSelVerts = VertsByNumber.count
						--format "CountOfSelVerts: %\n" CountOfSelVerts
					)
					
				
					polyop.setVertSelection CurrentObj SelectionOfVerts
					subobjectlevel = 1
					
				)
				else messagebox "No verts selected. Does this model have open edges?"
				--------------
				redrawViews()
			)
			

		)
		else messagebox "Model Zipper combines two halves of a mesh together.\nSelect the first half (Must be an Editable Poly object with an open seam) and then you will be able to pick and then weld to its counterpart." 
		
		

		
	)

	function zipModelsTogether selection PickedObj =
	(
		if VertCount != undefined do
		(
			with undo off
			(
				--------------
				if ZipPrefs.autohold do max hold
				--------------
				CurrentObj = selection[1]
				--
				UseUserSelection = true
				Tolerance = roll_automat_ModelZipper.spin_EdgeFudge.value
				getVertsEdge selection Tolerance --running this again with user's vert selection
				UseUserSelection = false
				
				--------------
				--------------
				VertsByNumber=#()
				for loop = 1 to VertCount do
				(
					if true == SelectionOfVerts[loop] do append VertsByNumber loop
				)
				CountOfSelVerts = VertsByNumber.count
				
-- 				VertsByNumber --defined by get verts on seam
-- 				CountOfSelVerts --count of same
-- 				VertCount --need to ensure objects match
				
				if polyop.getNumVerts PickedObj == VertCount then
				(

					PickedObj.material = CurrentObj.material
					
					if ( InstanceMgr.CanMakeObjectsUnique #(PickedObj) ) do InstanceMgr.MakeObjectsUnique &PickedObj #individual
					polyop.attach CurrentObj PickedObj
					
					for loop = 1 to CountOfSelVerts do
					(
						FirstVertex = VertsByNumber[loop]
						FirstVertPos = VertPos CurrentObj FirstVertex
						
						SecondVertex = VertsByNumber[loop] + VertCount
						SecondVertPos = VertPos CurrentObj SecondVertex
						
						midx = (FirstVertPos[1] + SecondVertPos[1] )/2
						midy = (FirstVertPos[2] + SecondVertPos[2] )/2
						midz = (FirstVertPos[3] + SecondVertPos[3] )/2
						
						polyop.setVert CurrentObj #( FirstVertex, SecondVertex ) [ midx, midy, midz ]
					)
					
					subobjectlevel = 1
					temp = CurrentObj.weldThreshold
					CurrentObj.weldThreshold = 0.0001
					
					polyop.weldVertsByThreshold CurrentObj #selection
					
					CurrentObj.weldThreshold = temp
					subobjectlevel = 0

				)
				else messagebox "Objects don't seem to match."
			)
		)
	)
	
	if true != automat_OpenScriptAsRollout do
	(
		try (closeRolloutfloater automat_ModelZipper ) catch ()
		automat_ModelZipper = newRolloutFloater "Model Zipper" 170 200
		addRollout roll_automat_ModelZipper automat_ModelZipper
		automat_automatModelZipper.size.y = ( roll_automat_ModelZipper.height + 30 )
	)
)