
--	ObjCleaner.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.0
--	CREATED:	12-19-2009 --3dsmax 2010


--Changelog

--3-22-2014	Added one time warning the first time user tries to clean obects.
--3-6-11 added ini file and made all settings sticky, added new icon with logic for handling gamma changes, added undo off context, adding undo warning to tooltio for clean button
(
	global automat_ObjCleaner
	--------------------------------------------------------
	
	if true != BurkeLibLoaded do include "$userScripts/BurkeStudios/lib/BurkeLib.ms" 
		
	local CurrentINI = GetDir #plugcfg + "\\Automat_ObjectCleaner.ini"
	
	local ObjArray
	local CleanedObjects = 0
	global automat_CleanMessage "-"
	local CleanColor = color 69 126 95
	local StickItOn = standard  Name: "Cleaned Objects"	DiffuseColor: CleanColor
	local G_CleanedSelection=#() --new empty array
	local tempbox
	
	local userhasbeenwarned = false
	--choose image based on gamma settings, catch is for older versions of max
-- 	try
-- 		(
-- 			local Icons = if IDisplayGamma.gamma >= 2  or fileInGamma >= 2 and #gamma == IDisplayGamma.colorCorrectionMode then "$icons\BURKE-Icons2_22_24i.bmp" 
-- 				else "$icons\BURKE-Icons2_1_24i.bmp"
-- 		)
-- 		catch ( local Icons = "$icons\BURKE-Icons2_1_24i.bmp"   )

	
	
	--Open Icons w/ proper gamma
	openBurkeIconsWithProperGamma()
	
	
	
	--Function Names
	--------------------------------------------------------
	local collectCleanSettings
	--------------
	struct ObjCleanPrefsStruct
	(
		pivotmode = true,
		mats = true,
		parent = true,
		child = true,
		layer = true,
		backcull = true,
		wire=true
	)
	local ObjCleanPrefs

	--UI
	--/////////////////////////////////////////////
	--------------------------------------------------------
	rollout roll_automat_ObjCleaner "Object Cleaner"
	(			
		group "Pivots, Bounding Boxes:"
		(
			Radiobuttons rad_CleanMode offset:[2,0]  labels:#( "Align Pivots and BBoxes", "Leave as is") default:2
		)
			
		group "Retain These Properties:"
		(
			Checkbox Check_Material "Material" checked:true tooltip:"Material Assignments." across:2
			Checkbox Check_Layers "Layer" checked:true tooltip:"Retain layer info."
			--
			Checkbox Check_Parent "Parent" checked:true tooltip:"Retain any links to parent object." across:2
			Checkbox Check_Children "Children" checked:true tooltip:"Retain child links."
			--
			Checkbox Check_BackCull "BackCull" checked:true tooltip:"Backface Culling" across:2 
			Checkbox Check_Wire "Wire Color" checked:true tooltip:"Wirecolor"

			
		)
				
		edittext CleanMessageDisplay text:"-" label:false readonly:true width:152 offset:[-12,0]  --Displays objects cleaned
		button but_helpCleaner "" height:24 across:2 offset:[-31,3] border:false \
			images:#(Icons,"$icons\BURKE-Icons2_24a.bmp", IconsInFile,2,2,2,2, true )
		button but_Clean "Clean Selected Objects" width:124 height:30 offset:[-18,0] tooltip:"This will clean and rebuild all selected geometry.\n\nThis operation is NOT undoable." 

		
		
		--------------
		on roll_automat_ObjCleaner open do
		(
			getapref = execute ( getINISetting CurrentINI "Cleaner Settings" "Pivots, Bounding Boxes" )
			if getapref != OK do rad_CleanMode.state = getapref
			--------------
			getapref = execute ( getINISetting CurrentINI "Retain Properties" "Materials" )
			if getapref != OK do Check_Material.checked = getapref
			--------------
			getapref = execute ( getINISetting CurrentINI "Retain Properties" "Parent Links" )
			if getapref != OK do Check_Parent.checked = getapref
			--------------
			getapref = execute ( getINISetting CurrentINI "Retain Properties" "Child Links" )
			if getapref != OK do Check_Children.checked = getapref
			--------------
			getapref = execute ( getINISetting CurrentINI "Retain Properties" "Layer Assignment" )
			if getapref != OK do Check_Layers.checked = getapref
			--------------
			getapref = execute ( getINISetting CurrentINI "Retain Properties" "Backface Culling" )
			if getapref != OK do Check_BackCull.checked = getapref
			--------------
			getapref = execute ( getINISetting CurrentINI "Retain Properties" "Wire Color" )
			if getapref != OK do Check_Wire.checked = getapref
			--------------
			getapref = execute ( getINISetting CurrentINI "Cleaner Settings" "userhasbeenwarned" )
			if getapref != OK do userhasbeenwarned = getapref
				
			
		)
		--------------
		on rad_CleanMode changed state do
		(
			setapref = setINISetting CurrentINI "Cleaner Settings" "Pivots, Bounding Boxes" ( state as string )
		)
		
		on Check_Material changed state do setapref = setINISetting CurrentINI "Retain Properties" "Materials" ( state as string )
		on Check_Parent changed state do setapref = setINISetting CurrentINI "Retain Properties" "Parent Links" ( state as string )
		on Check_Children changed state do setapref = setINISetting CurrentINI "Retain Properties" "Child Links" ( state as string )
		on Check_Layers changed state do setapref = setINISetting CurrentINI "Retain Properties" "Layer Assignment" ( state as string )
		on Check_BackCull changed state do setapref = setINISetting CurrentINI "Retain Properties" "Backface Culling" ( state as string )
		on Check_Wire changed state do setapref = setINISetting CurrentINI "Retain Properties" "Wire Color" ( state as string )

		
		
		--LOCAL Functions
		--------------------------------------------------------
		function collectCleanSettings =
		(
			ObjCleanPrefs = ObjCleanPrefsStruct pivotmode: ( if 2 == rad_CleanMode.state then true else false ) \
			mats:Check_Material.checked parent:Check_Parent.checked child:Check_Children.checked \
			layer:Check_Layers.checked backcull:Check_BackCull.checked wire:Check_Wire.checked
		)
		
		on but_Clean pressed do
		(
			
			if false == userhasbeenwarned then if querybox (  "This will collapse all of your selected objects to Editable Poly objects.
Only the object properties you've checked will be retained.\n\nContinue?"  )
			do
			(
				userhasbeenwarned = true
				setapref = setINISetting CurrentINI "Cleaner Settings" "userhasbeenwarned" ( userhasbeenwarned as string )
			)
			
			collectCleanSettings()
			--with undo off  ( cleanSelectionOfObjects Selection ObjCleanPrefs )
			with undo off  ( cleanSelectionOfObjects Selection ObjCleanPrefs )
			CleanMessageDisplay.text = automat_CleanMessage
		)
		
		--------------------------------------------------------
		on but_helpCleaner pressed do
		(
			lineonetext = "This tool is something like a Reset XForm except that it won't trash your models the way Reset XForm sometimes does.\n\n"
			linetwotext =  "It takes a selection of geometry objects and returns flattened Editable Polys. The objects are actually rebuilt so the transforms are new and clean.\n\n" 
			linethreetext = "Object names, materials, parent and child linkage, and layer info are restored. Object pivots and object bounding boxes can be kept or adjusted. All other object properties are stripped clean. \n\n"
			linefourtext = "This tool is ideal for cleaning up geometry before it is rigged or exported to another program or game engine."
			messagebox ( lineonetext + linetwotext + linethreetext + linefourtext )
		)
				
			
	) 
	

	if true != automat_OpenScriptAsRollout then
	(
		try (closeRolloutfloater automat_ObjCleaner ) catch ()
		automat_ObjCleaner = newRolloutFloater "Object Cleaner" 170 100
		addRollout roll_automat_ObjCleaner automat_ObjCleaner
		automat_ObjCleaner.size.y = ( roll_automat_ObjCleaner.height + 30 )
	)
	else()
	
	
)
