
--	ObjectTools.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.4
--	CREATED:	02-19-2011 --3dsmax 2011

--7-7-11	Add Bone Tools Section
--6-19-11	Object Naming Tools Added
--5-11-11	Added Implanting Tool
--5-3-11	INI settings for import settings checkboxes
--5-1-11	Import Directory tool can now rename objects by filename. Doesn't check for existing name conflicts.
--4-30-11	First pass at function to import an entire directory, very rough but works
--4-25-11	Added 'Export Objects to OBJs'
--9-1-11	Check Object Names started
--9-2-11	New LOD feature
(
	global automat_ObjectTools
	
	--------------------------------------------------------
	local CurrentINI = GetDir #plugcfg + "\\automat_ObjectTools.ini"
	--------------------------------------------------------
	if true != BurkeLibLoaded do include "$userScripts/BurkeStudios/lib/BurkeLib.ms"
	
	
	--------------------------------------------------------
	local AlignThoseX = false
	local AlignThoseY = false
	local AlignThoseZ = false
	--
	local bool_DeepOnOffEnabled = false
	local bool_SDivSetToThis
	

	
	--Function Prototypes
	--------------------------------------------------------
	local selectHalfOfObject
	local AlignSomeVertices
	--
	local setSubdivisonForSelection
	local setIsolineForSelection
	local setBackFaceCullOnOff
	
	--UI
	--/////////////////////////////////////////////
	--------------------------------------------------------
	rollout roll_automat_ObjectTools "Object Tools"
	(
		group "Editable Poly:"
		(
			label SelectLR "Select:" across:3 offset:[16,2]
			button but_SelectLeft "Left"\
			tooltip:"Left-click: Select faces on left side of model.\nRight-click: DELETE faces on left side of model.\n\nThis script is mostly intended to help select one half of a symmetrical model i.e. character model." \
			width:30 offset:[15,-2]
			
			button but_SelectRight "Right" \
			tooltip:"Left-click: Select faces on right side of model.\nRight-click: DELETE faces on right side of model.\n\nThis script is mostly intended to help select one half of a symmetrical model i.e. character model."  \
			width:30 offset:[4,-2]
			
			label VertsAlign "Align Verts:" across:4 offset:[10,2]
			button but_AlignVertsX "x" tooltip:"X value of all selected verts set to the average X value." width:20 offset:[26,-2]
			button but_AlignVertsY "y" tooltip:"Y value of all selected verts set to the average Y value." width:20 offset:[15,-2]
			button but_AlignVertsZ "z" tooltip:"Z value of all selected verts set to the average Z value." width:20 offset:[4,-2]
		)
		
		group "Object Properties:"
		(
			label SubdivisionToggler "Subdivision:" across:3 offset:[6,4]
			button but_SDivOn "on" tooltip:"Left-click: Enables SubD in E-Poly Object OR in any SubD modifiers\n\nRight-click: Enables SubD in E-Poly Objects AND in any SubD modifiers" width:20 offset:[10,0]
			button but_SDivOff "off" tooltip:"Left-click: Disables SubD in E-Poly Object OR in any SubD modifiers\n\nRight-click: Disables SubD in E-Poly Objects AND in any SubD modifiers" width:20 offset:[-12,0]
			
				
			label IsolineToggler "Isoline Disp:" across:3 offset:[6,2]
			button but_IsolineOn "on" tooltip:"Left-click: Enables Isoline Display in E-Poly Object OR in any SubD modifiers\n\nRight-click: Enables Isoline Display in E-Poly Objects AND in any SubD modifiers" width:20 offset:[10,-2]
			button but_IsolineOff "off" tooltip:"Left-click: Disables Isoline Display in E-Poly Object OR in any SubD modifiers\n\nRight-click: Disables Isoline Display in E-Poly Objects AND in any SubD modifiers" width:20 offset:[-12,-2]	
		)
		
		group "Remove Attributes:"
		(
			button but_MoveToOrigin "Translation" width:60 pos:[8,170]  tooltip:"Left-click: Move each object to Origin.\nRight-click: Center selection to origin.\nCtrl+Right-click: Stand selection on origin." 
			button but_RemoveRotation "Rotation"   width:48 pos:[68,170] tooltip:"Remove rotation from selected objects." 
			button but_RemoveScale "Scale"  width:33 pos:[116,170] tooltip:"Reset scale of selected objects to 100%." 
		)
		
		
		button but_FindInstances "Find Instances" width:78 pos:[4,204]\
		tooltip:"Finds and Selects all instances of currently selected objects. Objects in selection that have no instance will be deselected." 
		
		button but_MakeUnique "Make Unique" width:72 pos:[82,204] tooltip:"Remove all instancing from selection of objects." 
		
		
		
		
		button but_setCulling "BackFaceCull Off" width:90 pos:[4,225] tooltip: "Click to DISABLE backface culling for a selection of objects. If no objects are selected it will disable backface culling on ALL objects in scene.\n\nCtrl-Click will ENABLE backface culling." 
		button but_Combine "Combine" width:60 pos:[94,225] tooltip:"Collapse all selected objects into a single Editable Poly. First object selected is the one that remains. Much faster than the built-in max collapse tool." 

		--/////////////////////////////////////////////
		--OPEN
		--/////////////////////////////////////////////
		--------------------------------------------------------
		on roll_automat_ObjectTools open do
		(
		)

		
		--BUTTONS
		--/////////////////////////////////////////////
		--------------------------------------------------------
		
	
		--------------------------------------------------------
		on but_SelectLeft pressed do
		(
			selectHalfOfObject selection "left"
		)
		on but_SelectLeft rightclick do
		(	
			selectHalfOfObject selection "left"
			undo on
			(
				polyop.deleteFaces selection[1] #selection delIsoVerts:true
			)
		)
		
		--------------
		on but_SelectRight pressed do
		(
			selectHalfOfObject selection "right"
		)
		on but_SelectRight rightclick do
		(
			selectHalfOfObject selection "right"
			with undo on
			(
				polyop.deleteFaces selection[1] #selection delIsoVerts:true
			)
		)
		
		--------------
		on but_AlignVertsX pressed do
		(	
-- 			try
-- 			(
					if polyop.getVertSelection $ !=undefined and subobjectLevel == 1 do
				(
					BunchOfVerts = polyop.getVertSelection $
					AlignThoseX=true
					AlignSomeVertices (BunchOfVerts)
				)
-- 			) catch()
		) 
		
		--------------
		on but_AlignVertsY pressed do
		(	
			try
			(
				if polyop.getVertSelection $ !=undefined and subobjectLevel == 1 do
				(
					BunchOfVerts = polyop.getVertSelection $
					AlignThoseY=true
					AlignSomeVertices (BunchOfVerts)
				)
			) catch()
		)
		
		--------------
		on but_AlignVertsZ pressed do
		(	
			try
			(
				if polyop.getVertSelection $ !=undefined and subobjectLevel == 1 do
				(
					BunchOfVerts =polyop.getVertSelection $
					AlignThoseZ=true
					AlignSomeVertices (BunchOfVerts)
				)
			) catch()
		)
		
		--------------------------------------------------------
		--------------------------------------------------------
		on but_SDivOn pressed do
		(
			bool_DeepOnOffEnabled=false
			bool_SDivSetToThis=true --TURN IT OFF
			setSubdivisonForSelection selection --FUNCTION FOR SDiv 
		)
		on but_SDivOn rightclick do
		(
			bool_DeepOnOffEnabled=true
			bool_SDivSetToThis=true --TURN IT OFF
			setSubdivisonForSelection selection --FUNCTION FOR SDiv 
		)
		
			--------------
		on but_SDivOff pressed do
		(
			bool_DeepOnOffEnabled=false
			bool_SDivSetToThis=false --TURN IT OFF
			setSubdivisonForSelection selection --FUNCTION FOR SDiv 
		)
		on but_SDivOff rightclick do
		(
			bool_DeepOnOffEnabled=true 
			bool_SDivSetToThis=false --TURN IT OFF
			setSubdivisonForSelection selection --FUNCTION FOR SDiv 
		)
		
		--------------
		on but_IsolineOn pressed do
		(
			bool_DeepOnOffEnabled=false
			bool_SDivSetToThis=true
			setIsolineForSelection selection
		)
		on but_IsolineOn rightclick do
		(
			bool_DeepOnOffEnabled=true
			bool_SDivSetToThis=true
			setIsolineForSelection selection
		)
		
		--------------
		on but_IsolineOff pressed do
		(
			bool_DeepOnOffEnabled=false
			bool_SDivSetToThis=false
			setIsolineForSelection selection
		)
		on but_IsolineOff rightclick do
		(
			bool_DeepOnOffEnabled=true
			bool_SDivSetToThis=false
			setIsolineForSelection selection
		)
		
		--------------
		
		on but_MoveToOrigin pressed do
		(
			undo on
			(
				if selection.count > 0 do
				(
					$.position=[0,0,0]
				)
			)
		)
		on but_MoveToOrigin rightclick do
		(
			undo on
			(
				if selection.count > 0 do
				(
					beefy = selection.center
					beefyadjust = [0,0,0] - beefy
					move $ beefyadjust
						
					if keyboard.controlPressed do
					(
						zoffset = if selection.min.z < 0 then abs selection.min.z else 0
						move selection [0,0,	zoffset]
					)
				)
			)
		)
		
		--------------
		
		on but_RemoveRotation pressed do
		(
			ObjArray = for piggy in selection where true == isValidNode(piggy) collect piggy
			undo on
			(
				for EachObject in ObjArray do EachObject.rotation = (quat 0 0 0 0)
			)
		)
		
		--------------
		
		on but_RemoveScale pressed do
		(
			ObjArray = for piggy in selection where true == isValidNode(piggy) collect piggy
			undo on
			(
				for EachObject in ObjArray do EachObject.scale = [1,1,1]
			)
		)
		
		
		--------------
		
		on but_MakeUnique pressed do
		(
			ObjArray = for piggy in selection where true == isValidNode(piggy) collect piggy
			undo on
			(
				for EachObject in ObjArray do 
				(
					if ( InstanceMgr.CanMakeObjectsUnique #(EachObject) ) do InstanceMgr.MakeObjectsUnique &EachObject #individual
				)
			)	
		)
		
		--------------
		
		on but_Combine pressed do
		(
			with undo off
			with redraw off
			(
				
				ObjArray = for piggy in selection where superclassof(piggy) == GeometryClass collect piggy
				objcount = ObjArray.count
				if objcount > 0 do
				(
					
					restoremode = false
					if getCommandPanelTaskMode() == #modify do
					(
						restoremode = true
						max create mode
					)
					
					thename = ObjArray[1].name
					collapsedjunk = ConvertToPoly (  box position:ObjArray[1].pos name:( "tempAutomatobjectforcollapse") )
					polyop.deleteFaces collapsedjunk  #{1..collapsedjunk.GetNumFaces()}
					
					for each in ObjArray do
					(
						
							polyop.attach collapsedjunk each
						
					)
					collapsedjunk.name = thename
					clearSelection()
					select ( getNodeByName(thename))
						
					if restoremode do max modify mode
					
				)
			)	
		)
		
		
		on but_setCulling pressed do
		(
			state = keyboard.ControlPressed
				
			setBackFaceCullOnOff state
		)
		--------------
		
		
		on but_FindInstances pressed do
		(
			NewSelectionofObjectsArray = #()
			
			--this appends some items multiple times but it's probably faster than trying to remove duplicate objects from the set so whatevs
			for each in Selection do
			(
				InstanceMgr.GetInstances each &FoundInstances
				if FoundInstances.count > 1 do
				(
					for each in FoundInstances do append NewSelectionofObjectsArray each
				)
			)
			
			thecount = NewSelectionofObjectsArray.count
			if  thecount > 1
				then
				(
					select NewSelectionofObjectsArray
					
				)
				else 
				(
					max select none
					--popPrompt()
					--displayTempPrompt "No instances found. Nothing to select." 50
					
				)
		
		)
		
		
		
	)--rollout
	
	--FUNCTIONS
	--/////////////////////////////////////////////
	--------------------------------------------------------
	function selectHalfOfObject selection Target =
	(
		with redraw off
		(
			subobjectlevel = 0
			
			CurrentObj = selection[1]
			if Editable_Poly == classof ( CurrentObj.baseobject ) do
			(
				CurrVertSelection=#{}
				
				ObjCenter = ( CurrentObj.position.x )
				
				CurrVertCount = CurrentObj.numVerts
				
				--/////////////////////////////////////////////
				--Calculate CenterLine
				CenterLine = 100
				
				--Finds the closest vert to the calculated center of the object
				for loopy = 1 to CurrVertCount do
				(
					CurrentX = (VertPos CurrentObj loopy).x
					CurrDiff = abs ( CurrentX - ObjCenter )
					OldDiff = abs ( CenterLine - ObjCenter )
					if CurrDiff < OldDiff do CenterLine = CurrentX
				)
-- 				format "x.CenterLine: %\n" CenterLine
-- 				format "x.ObjCenter: %\n" ObjCenter
				--/////////////////////////////////////////////
				
				
				
				for loop = 1 to CurrVertCount do
				(
					CurrentX = (VertPos CurrentObj loop).x 
					if CurrentX > CenterLine then CurrVertSelection[loop] = true
						else if ( close_enough CurrentX CenterLine 10 ) then CurrVertSelection[loop] = true
							else CurrVertSelection[loop] = false
					
	-- 				if loop == 91 do
	-- 				(
	-- 					format "loop: %\n" loop
	-- 					format "CurrentX: %\n" CurrentX
	-- 					format "CenterLine: %\n" CenterLine
	-- 					format "CurrVertSelection[loop]: %\n" CurrVertSelection[loop]
	-- 				)
					
				)
				
				max modify mode
				subobjectlevel = 1
				polyop.setVertSelection CurrentObj CurrVertSelection
				max select invert
				CurrFaceSelection = polyop.getFacesUsingVert CurrentObj #selection
				
				--CurrFaceSelection = polyop.getFacesUsingVert CurrentObj CurrVertSelection
				subobjectlevel = 4
				polyop.setFaceSelection CurrentObj CurrFaceSelection
				
				if "right" == Target  do max select invert
				
				
			)
		)
	)
	
	--------------------------------------------------------
	function AlignSomeVertices BunchofVerts =
	(
		undo on
		(
			SelectedX=#() --create arrays for storing x,y,z values of each vertex in selection
			SelectedY=#()
			SelectedZ=#()
			
			For count in BunchOfVerts do -- this gathers up all the  xyz values and stores them in separate arrays
			(
				Append SelectedX (polyop.getvert $ count).x
				Append SelectedY (polyop.getvert $ count).y
				Append SelectedZ (polyop.getvert $ count).z
				
			)
			
			NewX=(((amax SelectedX)+(amin SelectedX))/2) --sets NewX value to midpoint between minimum and maximum in array values
			NewY=(((amax SelectedY)+(amin SelectedY))/2) --sets NewY value to midpoint between minimum and maximum in array values
			NewZ=(((amax SelectedZ)+(amin SelectedZ))/2) --sets NewZ value to midpoint between minimum and maximum in array values
			
			
			For count in BunchOfVerts do
			(
				TempX=(polyop.getvert $ count).x
				TempY=(polyop.getvert $ count).y
				TempZ=(polyop.getvert $ count).z
				
				if AlignThoseX == true do polyop.setvert $ count [NewX,TempY,TempZ]
				if AlignThoseY == true do polyop.setvert $ count [TempX,NewY,TempZ]
				if AlignThoseZ == true do polyop.setvert $ count [TempX,TempY,NewZ]
			)
	
			AlignThoseX = false --reset variables, switched back on by pressing appropriate buttons
			AlignThoseY = false
			AlignThoseZ = false
		)
		
	)
	
	--------------------------------------------------------
	function setSubdivisonForSelection selection =
	(
		objArray = for piggy in selection where classof(piggy) == Editable_Poly \
			and piggy.modifiers.count == 0 collect piggy	--EPOLY only

		for EachObject in objArray do EachObject.surfSubdivide = bool_SDivSetToThis --SET VALUE
		
		objArray = for piggy in selection where superclassof(piggy) == GeometryClass \
			and piggy.modifiers.count != 0 collect piggy	--GEOM W/ MODIFIERS
		
		for i = 1 to objArray.count do
			(
				SDivModsFoundInStack=0 --COUNTER
				
				for j = 1 to objArray[i].modifiers.count do
				(
					if classof(objArray[i].modifiers[j]) == TurboSmooth or\
					classof(objArray[i].modifiers[j]) == MeshSmooth or\
					classof(objArray[i].modifiers[j]) == HSDS do
					(
						objArray[i].modifiers[j].enabled=bool_SDivSetToThis --ENABLE OR DISABLE MODIFIER
						SDivModsFoundInStack+=1 --ADD COUNT FOR EACH SDIV MOD FOUND
					)
				)
				
				if bool_DeepOnOffEnabled==true or SDivModsFoundInStack==0 do --SET EVERYTHING
					(
						ObjUnderModStack=objArray[i].baseobject --GET BASE OBJ
						if classof(ObjUnderModStack)==Editable_Poly do \
							ObjUnderModStack.surfSubdivide=bool_SDivSetToThis --EPOLY
						
						--ADD EMESH HERE
					)
				
				
				--NO CHECKING FOR EDITABLE POLY SMOOTHING
			)
			bool_DeepOnOffEnabled=false --RESET
			--redrawViews()
			select Selection --GET A GOOD REDRAW
	)
				
			
	
	function setIsolineForSelection selection =
	(
		objArray = for piggy in selection where classof(piggy) == Editable_Poly \
			and piggy.modifiers.count == 0 collect piggy	--EPOLY only

		for EachObject in objArray do EachObject.isolineDisplay=bool_SDivSetToThis --SET VALUE
		
		objArray = for piggy in selection where superclassof(piggy) == GeometryClass \ --GEOM W/ MODIFIERS
			and piggy.modifiers.count != 0 collect piggy	
		
		for i = 1 to objArray.count do
			(
				SDivModsFoundInStack=0 --COUNTER
				
				for j = 1 to objArray[i].modifiers.count do
				(
					if classof(objArray[i].modifiers[j]) == TurboSmooth or\
					classof(objArray[i].modifiers[j]) == MeshSmooth do
					(
						objArray[i].modifiers[j].isolineDisplay=bool_SDivSetToThis --ENABLE OR DISABLE MODIFIER
						SDivModsFoundInStack+=1 --ADD COUNT FOR EACH SDIV MOD FOUND
					)
				)
				
				if bool_DeepOnOffEnabled==true or SDivModsFoundInStack==0 do --SET EVERYTHING
					(
						ObjUnderModStack=objArray[i].baseobject --GET BASE OBJ
						if classof(ObjUnderModStack)==Editable_Poly do \
							ObjUnderModStack.isolineDisplay=bool_SDivSetToThis --EPOLY
						
						--ADD EMESH HERE
					)
				
				--NO CHECKING FOR EDITABLE POLY SMOOTHING
			)
			bool_DeepOnOffEnabled=false --RESET
			--redrawViews()
			select Selection --GET A GOOD REDRAW
	)
	
	function setBackFaceCullOnOff state =
	(

		ObjArray = for each in Selection where superclassof (each.baseobject) == geometryclass collect each
		
		queryword = ( state as string )
		
		querytext = "No objects selected." +"\n\n" + "Set backface culling to " + queryword + " for all objects in scene?"
		
		if ObjArray.count == 0 then  if  ( querybox  querytext ) do
		(
			ObjArray = for each in Objects where superclassof (each.baseobject) == geometryclass collect each
		)
		
		
		if ObjArray.count != 0 then
		(
			popPrompt()
			
			for each in ObjArray do
			(
				each.backfaceCull = state
			)
			
			thetext = "Backface culling set to " + queryword +  " for " + (ObjArray.count as string) + " geometry objects."
			
			displayTempPrompt thetext 2000
		
		)
		else 
		(
			displayTempPrompt "No Geometry found." 2000
		)
		
	)
	
	
	--------------------------------------------------------
	if true != automat_OpenScriptAsRollout do
	(
		try (closeRolloutfloater automat_ObjectTools ) catch ()
		automat_ObjectTools = newRolloutFloater "Object Tools" 170 90
		addRollout roll_automat_ObjectTools automat_ObjectTools
		automat_ObjectTools.size.y = ( roll_automat_ObjectTools.height + 30 )
	)
	
)
