
--	SpinScatter.ms
--	Steve Burke
--	www.burkestudios.com

--	version 1.2

--	CREATED:	02-19-2011 --3dsmax 2011

--revision highlights
--------------
-- 2-5-12	added store/load settings functions and interface, improved interface.
--	3/3/11		added Do for Root Object Section and related functions/variables

(
	global automat_SpinScatter
	
	if true != BurkeLibLoaded do include "$userScripts/BurkeStudios/lib/BurkeLib.ms"
		
	--Open Icons w/ proper gamma
	openBurkeIconsWithProperGamma()
	
	
	local CurrentINI = GetDir #plugcfg + "\\Automat_SpinScatter.ini"
	local RolloutPosition= [200,200]
	
	getapref = execute ( getINISetting CurrentINI "User Settings" "RolloutPosition" )
			if getapref != OK do RolloutPosition = getapref
	
	--------------------------------------------------------
	global	automat__GoldSpinObjArray
	local		CenterOfSpin
	local		automat_SpinObjInfoArray=#()
	global 	GoldSpinPrefs
	local 	JustChangePositions = false
	local		UpdatedSpinPosAndScale
	local		LastSpinObject
	local		setRootRenderState
	local WeBeSpinningNow =  false
	--
	global		automat_count, automat_loop, automat_ease, automat_UserValue, automat_xsize, automat_ysize, automat_zsize, automat_mode, automat_rebuild = false, automat_maxcount
	global		automat_SpinObj, automat_NewObj, automat_tempcenterobj, automat_RadiusInc
	
	--Functions
	
	local getUISettings
	local turnSettingsInterfaceOffOn
	local getCategoriesFromIniFile
	local saveIniSettings
	local updateSettingsUI
	local storeSettings
	local loadSettingsForThisCatAndKey
	
	
	--------------------------------------------------------
	--------------------------------------------------------
	--Function Names
	--------------------------------------------------------
	local spinObjectAtGoldenAngle
	
	local unflagSpinNodes
	
	local deleteAndSpinAgain
	local dontDeleteAndSpinAgain
	local deleteLastGoldSpins
	local selectLastGoldSpins
	local collpaseLastGoldSpins
	local selectEveryNGoldSpins
	
	local saveKeyNumber
	
	local storeSettings
		local AllCatsArray =#("--")
		local AllKeys_MD_Array =#("--")
		
		--
		local CurrCat = 0
		local CatCount = 0
		
		-- When set to 0 it will mean there are no keys for this category.
		local CurrKey = 0
		local KeyCountArray =#()
	
	local struct_GoldAngleScatterPref
	
	local editCategoryInfo
		local EditCat_RolloutPosition = [ 200,200 ]
		local roll_automat_EditCatName
		local automat_EditCatName
		local EditCat_TextFillString
		local EditCat_WindowTitle "Edit Categroy Rollout"
		local EditCat_RenameMode = false
		
	local LoadLock = false
		
	local renameIniSettings
	
	local deleteSettings
	
	--------------

	try ( automat__GoldSpinObjArray[1] ) catch ( automat__GoldSpinObjArray=#() ) --holds object created with Gold spin

	--------------------------------------------------------
	struct struct_GoldAngleScatterPref
	(
		Count = 144 ,
		BuildPercent = 100,
		Growth = 100 ,
		Padding = 50 ,
		GAngle = 137.5 ,
		PadStart = 0,
		ZDrop = 0,
		Tilt = 0,
		EaseAmount = 1,
		GThin = false,
		ThinAmount = 20,
		SeedNum = 13528,
		UseCustom = false,
		StartCustom = 50,
		Expression = "",
		UserValue = 1,
		DupMode = 1,
		RenderRoot = 1
	)

	--------------------------------------------------------
	local SpinExpressionStrings=#
	(
		"/*adjust spacing*/ automat_RadiusInc -= ( ( automat_UserValue/100 * automat_ease ) )",
		"/*small to big*/ automat_SpinObj.scale = [ nim = automat_UserValue * automat_ease, nim, nim ]",
		"/*scale*/ automat_SpinObj.scale = [ nim =  1 - ( automat_UserValue * automat_ease ) , nim, nim ]",
		"/*shift X*/ move automat_SpinObj  [automat_UserValue *  automat_xsize * automat_ease , 0 , 0 ]",
		"/*shift  Z*/ move automat_SpinObj  [ 0, 0, automat_UserValue / 10 *  automat_zsize * automat_ease ]", 
		"/*change spin angle*/ in coordsys automat_tempcenterobj ( rotate automat_tempcenterobj ( quat  ( automat_UserValue / 10 ) z_axis ) )",
		"/*scale Z*/ in coordsys automat_SpinObj ( automat_SpinObj.scale = [ 1, 1, 1 + ( automat_UserValue * automat_ease ) ] )",
		"/*wraparound*/ in coordsys automat_tempcenterobj ( rotate automat_SpinObj ( quat  automat_UserValue x_axis ) )",
		"/*wrap n scale*/ in coordsys automat_tempcenterobj ( rotate automat_SpinObj ( quat  automat_UserValue x_axis ) ); automat_SpinObj.scale = [ nim =  1 - ( automat_UserValue / 10 * automat_ease ) , nim, nim ]",
		"/*snakey*/ in coordsys local ( rotate automat_SpinObj ( quat  automat_UserValue ( normalize [ 3, 4, 5 ] ) ) )",
		"/*Jiggle*/ move automat_SpinObj  [ (random 0 1) * automat_UserValue *.10 *  automat_xsize * automat_ease , (random 0 1) * automat_UserValue*.10 *  automat_ysize * automat_ease , 0 ]",
		"/*COPYreq. bend*/if 2==automat_mode and automat_count<=automat_maxcount and isValidNode automat_NewObj do( max create mode; addModifier automat_NewObj (Bend angle:(automat_UserValue*automat_ease) axis:2 direction:-90); automat_rebuild=true) "
	)
		
	--------------------------------------------------------
	--------------------------------------------------------
	--------------------------------------------------------
	
	--------------------------------------------------------
	rollout roll_automat_SpinScatter "Spin Scatter"
	(
		group "Settings:"
		(
			spinner spin_Count "*Count:" range:[1,10000,144] type: #integer tooltip:"Number of copies to make"
			spinner spin_Angle "Angle:" range:[-360,360,137.5] tooltip:"Make an instance of the object every n degrees"
			--------------
			spinner spin_Growth "Growth %:" range:[-100,100,33] type: #integer tooltip:"Difference in scale from first object created to last."
			spinner spin_PadStart "Pad at Start:" range:[-2000,2000,5] tooltip:"Amount to offset objects from center of spin."
			spinner spin_Padding "Spacing:" range:[-2000,2000,5] tooltip:"Space between objects."
			spinner spin_Tilt "*Curvature:" range:[-360,360,0] tooltip:"Adds rotation along second axis."
			spinner spin_EaseAmount "Ease Curve:" range:[-25,25,1] tooltip:"Affect shape of curve."
			spinner spin_ZDrop "Z Spread:" range:[-5000,5000,0] type: #integer tooltip:"Vertical spacing between objects."
			spinner spin_Percent "*Build %:" range:[0,100,100] type: #integer tooltip:"Can build less than 100% of parts to add randomness."
		
			label lab_spinran "Build Percentage Settings:" align: #left 
			spinner spin_ThinAmount "%:" range:[0,99,20] type: #integer offset:[-16,0] across:2 enabled:false tooltip:"Percent to Thin Center. Objects are closer together so it is sometimes good to have less objects here."
			checkbox check_spinThin "Thin Center" offset:[-7,0] checked:true enabled:false tooltip:"Enable making less objects at center of spin; uses spinner to left."
			spinner spin_Seed "Seed:" range:[0,100000,13528] type: #integer enabled:false tooltip:"Only used when Build% is less than 100%"
			
			--------------
			button but_resetSpin "r"  height:16 across:2 offset:[-20,3] tooltip:"Restore default scatter settings."
			checkbutton but_spinObject "Scatter Object" width:100 height:30 offset:[-19,0]\
			tooltip:"Left-click: Scatter selected object.\nRight-click: Delete previous scatters.\n\nWhen this button is highlighted, changing the settings spinners will dynamically update the resulting scatter. Settings with an asterisk redo the scatter as opposed to just updating object positions; only important if you are making morph targets from scatters."
		)
		--------------
		group "Duplicate Mode:"
		(
			radiobuttons radio_DupMode "" labels:#("Instance", "Copy") tooltip:"Copy mode is slow but for some of the advanced effects it is necessary."
		)
		
		
		
		group "Do for Last Spin:"
		(
			button but_SelSpin "Select" width:40 offset:[-1,0] across:3 tooltip:"Select the last group of objects created."
			button but_DelSpin "Delete" width:40 offset:[-1,0] tooltip:"Delete last group of objects created. Leaves original object intact."
			button but_CollapseSpin "Collapse" width:50 offset:[4,0] tooltip:"Collapses the last group of object spun into a single Editable Poly."
			--button but_SelSpinN "Select Every n" align: #left across:2
			spinner spin_EveryN "Select Every N:" range:[0,1000,0] width:120 offset:[22,0] type: #integer tooltip:"Less objects will be created with gaps where objects would have gone."
		)
		group "Do for Root Object:"
		(
			button but_SelRoot "Select" width:40 offset:[-2,3] across:2 tooltip:"Select the original object at center of spin."
			label lab_RenderRoot "Render:" width:40 offset:[3,-4] 
			radiobuttons radio_RenderRoot "" labels:#("on", "off") offset:[34,-19] tooltip:"Disable rendering for original object and set object to display in XRay mode."
			
		)
		group "Store/Load Settings:"
		(
			button but_NewCategory "new"  offset:[51,0] width:32 enabled:true across:1 tooltip:"You can save your spins and organize by category names."
			--button but_EditCategory "edit"  offset:[18,0] width:32 enabled:true
			--
			dropDownList drop_SettingsName "Category:" items:#("empty") width:136  offset:[-3,-20] 
			--
			button but_KeyFrame "Store" width:48 height:32  offset:[1,6] across:3 tooltip:"saves spin settings"
			checkButton chb_LoadCurrent "Load" width:50 height:32 offset:[18,6] enabled:true tooltip:"load spin settings from disk\n\nNOTE: Object dimensions and pivot position have a great impact on the look of a spin. Different object using the same settings can look very dissimilar."
			checkButton chb_Lock "L" width:20 height:32 offset:[11,6] enabled:true tooltip:"Autolad. Will automatically load settings as you click through the arrows."
			
			
			button but_KeyLeft width:24 height:24 border:false enabled:true  offset:[5,0] \
				images:#(Icons,"$icons\BURKE-Icons2_24a.bmp", IconsInFile,13,13,14,14, true ) across:3 tooltip:"previous"
			
			editText edit_CurrentFrame "" width:45 text:"-" offset:[-2,3]
			
			button but_KeyRight width:24 height:24 border:false enabled:true  offset:[-2,0] \
				images:#(Icons,"$icons\BURKE-Icons2_24a.bmp", IconsInFile,11,11,12,12, true ) across:3 tooltip:"next"
			
			
			label lab_DeleteStuff "Delete the following:" 
			
			button but_DelCurrent "Current Key" width:62 across:2 offset:[0,0] enabled:false tooltip:"remove setting from disk"
			button but_DelCategory "Curr. Category" width:79 offset:[4,0] enabled:false tooltip:"remove category from disk"
			
			
		)
		group "Advanced:"
		(
			checkbox check_UseCustom "Use Custom Expression" offset:[0,0] checked:false tooltip:"adds additional variation to spins"
			spinner spin_StartCustom "Start at loop%:" range:[0,99,50] width:100 type: #integer align: #right enabled:false
			spinner spin_UserValue "UserValue:" range:[-1000,1000,1] enabled:false
			--spinner spin_UserVariable "User Value:" range:[1,10000,0] type: #integer
			combobox combo_Expression "Custom expression:" items:SpinExpressionStrings enabled:false 
			
		)
		
		
		on roll_automat_SpinScatter open do
		(
			roll_automat_SpinScatter.getUISettings()
			getCategoriesFromIniFile()
			updateSettingsUI CurrCat CurrKey true
			
			
		)
		
		on roll_automat_SpinScatter close do
		(
			saveIniSettings()
			LastSpinObject = undefined --to get it to stop trying to use old data
			

		)
		
		on roll_automat_SpinScatter moved ScreenPos do
		(
			setapref =  setINISetting CurrentINI "User Settings" "RolloutPosition" (ScreenPos as string )
		)
		
		on drop_SettingsName selected state do
		(
			saveKeyNumber CurrCat
			--
			--if CurrCat != state do resetbool = true
			CurrCat = state
			
			updateSettingsUI CurrCat CurrKey true
			
			--if we are loading settings then do this stuff
			if true == LoadLock then 
				(
					loadSettingsForThisCatAndKey CurrCat CurrKey
					if true == WeBeSpinningNow then deleteAndSpinAgain()
				)
			
		)
		
		on chb_Lock changed state do
		(
			LoadLock = chb_LoadCurrent.state = state
			
			
		)
		
		
		on but_KeyFrame pressed do
		(
			--We add a key
			CurrKey += 1
			KeyCountArray[CurrCat] += 1
			
			--next we call function to assign the current prefs to the current Category and Key Number
			storeSettings CurrCat ( KeyCountArray[CurrCat] )
			updateSettingsUI CurrCat CurrKey true
		)
		
		on but_KeyLeft pressed do
		(
			if CurrKey > 1 do
			(
				CurrKey -= 1
				updateSettingsUI CurrCat CurrKey false
				--
				if true == LoadLock then 
				(
					loadSettingsForThisCatAndKey CurrCat CurrKey
					if true == WeBeSpinningNow then deleteAndSpinAgain()
				)
			)			
		)
		
		on but_KeyRight pressed do
		(
			if CurrKey < KeyCountArray[CurrCat] do
			(
				CurrKey += 1
				updateSettingsUI CurrCat CurrKey false
				--
				if true == LoadLock then 
				(
					loadSettingsForThisCatAndKey CurrCat CurrKey
					if true == WeBeSpinningNow then deleteAndSpinAgain()
				)
			)			
		)
		
		on chb_LoadCurrent changed state do
		(
			
			loadSettingsForThisCatAndKey CurrCat CurrKey
			chb_LoadCurrent.checked = false
			if true == WeBeSpinningNow then deleteAndSpinAgain()
		)
		--
		on but_NewCategory pressed do
		(
			EditCat_RenameMode = false
			EditCat_WindowTitle = "Create New Category"
			EditCat_TextFillString = ""
			--
			editCategoryInfo  EditCat_WindowTitle EditCat_TextFillString EditCat_RenameMode
		)
		
		on but_EditCategory pressed do
		(
			EditCat_RenameMode = true
			EditCat_WindowTitle = "Edit Category Name"
			EditCat_TextFillString = AllCatsArray[CurrCat]
			--
			editCategoryInfo EditCat_WindowTitle EditCat_TextFillString EditCat_RenameMode
		)
		
		on but_DelCurrent pressed do
		(
			deleteSettings CurrCat CurrKey "key"
			
			if true == LoadLock then 
				(
					loadSettingsForThisCatAndKey CurrCat CurrKey
					if true == WeBeSpinningNow then deleteAndSpinAgain()
				)
		)
		
		on but_DelCategory pressed do
		(
			deleteSettings CurrCat CurrKey "Cat"
			
			if true == LoadLock then 
				(
					loadSettingsForThisCatAndKey CurrCat CurrKey
					if true == WeBeSpinningNow then deleteAndSpinAgain()
				)
		)
		
		--------------
		local SpinVal
	
		--------------
		function enableSpinRandomUI SpinVal =
		(
			if SpinVal < 100 then
			(
				check_spinThin.enabled = true
				spin_ThinAmount.enabled = true
				spin_Seed.enabled = true
			)					
				else
			(
				check_spinThin.enabled = false
				spin_ThinAmount.enabled = false
				spin_Seed.enabled = false
			)
		)

		function getUISettings =
		(
			GoldSpinPrefs = struct_GoldAngleScatterPref Count:spin_Count.value Growth:spin_Growth.value \
			Padding:spin_Padding.value GAngle:spin_Angle.value PadStart:spin_PadStart.value \
			ZDrop:spin_ZDrop.value BuildPercent:spin_Percent.value Tilt:spin_Tilt.value \
			GThin:check_spinThin.checked ThinAmount:spin_ThinAmount.value SeedNum:spin_Seed.value \
			UseCustom:check_UseCustom.checked StartCustom: ( spin_StartCustom.value/100.0 ) \
			Expression: combo_Expression.text  UserValue: spin_UserValue.value DupMode: radio_DupMode.state \
			RenderRoot: radio_RenderRoot.state EaseAmount: ( spin_EaseAmount.value )
			
		)
		
		
		--------------
		on but_spinObject changed state do
		(
			WeBeSpinningNow = state
			--
			if true == state then
			(
				automat__GoldSpinObjArray=#()
				--
				getUISettings()
				--
				LastSpinObject = selection[1]
				setRootRenderState GoldSpinPrefs.RenderRoot
				--
				spinObjectAtGoldenAngle LastSpinObject GoldSpinPrefs
				spin_EveryN.value = 0
			)
			else
			(
				--spinning has stopped
			)
			
		)
		on but_spinObject rightclick do
		(
			if keyboard.controlPressed then 
			(
				boing = box()
				select boing
				q = quat 45 z_axis
				rotate boing q
			)
			else deleteLastGoldSpins()
			try ( select LastSpinObject) catch()
		)
		
		--------------
		on but_delSpin pressed do
		(
			if undefined != GoldSpinPrefs do deleteLastGoldSpins()
		)
		
		--------------
		on but_selSpin pressed do
		(
			if undefined != GoldSpinPrefs do selectLastGoldSpins()
		)
		
		--------------
		on spin_EveryN changed EveryN do
		(
			if undefined != GoldSpinPrefs do
			(
				EveryN = spin_EveryN.value
				selectEveryNGoldSpins EveryN
			)
		)
		
		--------------
		on but_CollapseSpin pressed do
		(
			if undefined != GoldSpinPrefs do collpaseLastGoldSpins()
		)
		
		--------------
		function resetSpinSettings =
		(
			spin_Count.value = 144
			spin_Angle.value = 137.5
			spin_Growth.value = 33
			spin_PadStart.value = 5
			spin_Padding.value = 5
			spin_Tilt.value = 0
			spin_EaseAmount.value = 1
			spin_Percent.value = 100
			spin_ZDrop.value = 0
			check_spinThin.checked = true
			spin_ThinAmount.value = 20
			spin_Seed.value = 13528
			spin_EveryN.value = 0
			--
			--
			--getUISettings()
			enableSpinRandomUI 100
		)
		
		--------------
		on but_resetSpin pressed do
		(
			if undefined != GoldSpinPrefs then
			(
				deleteLastGoldSpins()
				resetSpinSettings()
				getUISettings()
				deleteAndSpinAgain()
				unflagSpinNodes()	
			)
			else
			(
				deleteLastGoldSpins()
				resetSpinSettings()	
			)

		)	
		
	

		--interactive bits
		--/////////////////////////////////////////////
		--------------------------------------------------------
		
		--------------------------------------------------------
		on radio_DupMode changed state do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.DupMode = radio_DupMode.state
				deleteAndSpinAgain()
				unflagSpinNodes()
			)
		)
		
		on spin_Count changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.Count = NewValue
				deleteAndSpinAgain()
			)
		)
		--------------
		on spin_Angle changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.GAngle = NewValue
				dontDeleteAndSpinAgain()
			)
		)
		--------------
		on spin_Growth changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.Growth = NewValue
				dontDeleteAndSpinAgain()
			)
		)
		--------------
		on spin_PadStart changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.PadStart = NewValue
				dontDeleteAndSpinAgain()
			)
		)
		--------------
		on spin_Padding changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.Padding = NewValue
				dontDeleteAndSpinAgain()
			)
		)
		--------------
		on spin_Tilt changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.Tilt = NewValue
				deleteAndSpinAgain()
			)
		)
		--------------
		on spin_EaseAmount changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.EaseAmount = ( NewValue )
				dontDeleteAndSpinAgain()
			)
		)
		--------------
		on spin_ZDrop changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.ZDrop = NewValue
				dontDeleteAndSpinAgain()
			)
		)
		--------------
		on spin_Percent changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.BuildPercent = NewValue				
				deleteAndSpinAgain()
			)
			enableSpinRandomUI NewValue
		)
		--------------
		on spin_ThinAmount changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				enableSpinRandomUI NewValue
				--
				if true == check_spinThin.checked do
				(
					GoldSpinPrefs.ThinAmount = NewValue
					dontDeleteAndSpinAgain()
				)
				
			)
		)
		
		--------------
		on spin_Seed changed NewValue do
		(
			if true == WeBeSpinningNow do
			(

				GoldSpinPrefs.SeedNum = NewValue
				dontDeleteAndSpinAgain()
				
			)
		)
		
		--------------
		on but_SelRoot pressed do
		(
			try ( select LastSpinObject ) catch( messagebox "Nothing to Select'" )
		)
		
		--------------
		on radio_RenderRoot changed state do
		(
			if undefined != LastSpinObject do setRootRenderState state
			
		)
		
		--------------
		on check_UseCustom changed NewValue do
		(
			if true == WeBeSpinningNow do
			(

				GoldSpinPrefs.UseCustom = NewValue
				GoldSpinPrefs.Expression = combo_Expression.text
				GoldSpinPrefs.StartCustom = ( spin_StartCustom.value/100.0 )
				dontDeleteAndSpinAgain()
				--
			)
				if false == NewValue then				
				(
					spin_StartCustom.enabled = false
					combo_Expression.enabled = false
					spin_UserValue.enabled = false
				)
				else
				(
					spin_StartCustom.enabled = true
					combo_Expression.enabled = true
					spin_UserValue.enabled = true
				)
				
			
		)
		
		--------------
		on spin_StartCustom changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.StartCustom = ( spin_StartCustom.value/100.0 )
				dontDeleteAndSpinAgain()	
			)
		)
				--------------
		
		on combo_Expression doubleClicked  NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.UseCustom = true
				GoldSpinPrefs.Expression = combo_Expression.text
				GoldSpinPrefs.StartCustom = ( spin_StartCustom.value/100.0 )
				GoldSpinPrefs.UserValue = spin_UserValue.value
				dontDeleteAndSpinAgain()
			)
		)
		--------------
		on combo_Expression entered NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.UseCustom = true
				GoldSpinPrefs.Expression = NewValue
				GoldSpinPrefs.StartCustom = ( spin_StartCustom.value/100.0 )
				GoldSpinPrefs.UserValue = spin_UserValue.value
				dontDeleteAndSpinAgain()
			)
		)
		
		on spin_UserValue changed NewValue do
		(
			if true == WeBeSpinningNow do
			(
				GoldSpinPrefs.UserValue =  spin_UserValue.value
				
				if true == automat_rebuild then deleteAndSpinAgain()
					else dontDeleteAndSpinAgain()	
				
			)
		)
		
		
		--------------------------------------------------------
		on spin_Count buttonup  do unflagSpinNodes()
		on spin_Angle buttonup  do unflagSpinNodes()
		on spin_Growth buttonup  do unflagSpinNodes()
		on spin_PadStart buttonup  do unflagSpinNodes()
		on spin_Padding buttonup  do unflagSpinNodes()
		on spin_Tilt buttonup  do unflagSpinNodes()
		on spin_EaseAmount buttonup  do unflagSpinNodes()
		on spin_ZDrop buttonup  do unflagSpinNodes()
		on spin_Percent buttonup  do ( unflagSpinNodes(); redrawViews() )
		on spin_ThinAmount buttonup  do unflagSpinNodes()
		--
		on spin_StartCustom buttonup  do unflagSpinNodes()
		on spin_UserValue buttonup  do unflagSpinNodes()
		
		
		
		
		
	)--ROLLOUT
	
	--------------------------------------------------------
	--------------------------------------------------------
	--Small Rollout for Entering Category Name
	--------------------------------------------------------
	
	rollout roll_automat_EditCatName "Category Name" width:100 height:50
	(
		editText user_CatName "Name:" 
		
		button but_OK  "OK" pos:[112,35] width:64 height:32
		button but_Cancel "cancel" pos:[62,35] width:50 height:32
		--
		
		on roll_automat_EditCatName open do
		(
			--if we are editing a name the fill the textfield, otherwise leave it blank for a new categroy
			user_CatName.text = EditCat_TextFillString

		)
		--
		
		on roll_automat_EditCatName moved ScreenPos do
		(
			setapref =  setINISetting CurrentINI "Edit Categroy Rollout" "RolloutPosition" (ScreenPos as string )
		)
		--
		
		on but_Cancel pressed do
		(
			try( closeRolloutfloater automat_EditCatName ) catch()
		)
		--
		
		on but_OK pressed do
		(
			
			getthetext = user_CatName.text
			
			if getthetext != "" then
			(
				
				--gonna append because it is a quick way to see if the name exists
				appendIfUnique AllCatsArray getthetext
				
				if AllCatsArray.count > CatCount then
				(
					if true == EditCat_RenameMode then
					(
						--rename given category name in array of names
						
						--delete the appended category
						deleteItem AllCatsArray ( CatCount + 1 )
						
						--rename current category
						oldname = AllCatsArray[CurrCat]
						AllCatsArray[CurrCat] = getthetext
						
						--need to rename old data in the ini
						--yah, i really should do that
						renameIniSettings oldname getthetext
						
					)
					else
					(
						--if the count is bigger the name is unique and was added to the categories. Success.
						--write the new name to the ini file
						CatCount += 1
						CurrCat = CatCount
					
					)
					
					saveIniSettings()
					updateSettingsUI CurrCat 0 true
					try( closeRolloutfloater automat_EditCatName ) catch()
					
					
					

				)
				else
				(
					--category already exists, do nothing
					messagebox "There is already a category by that name."
				)
				
			)--End check for "" empty string
		
			
		)--EN D OK Button
		
		
	) --END ROLLOUT EditCatName
	
	--------------------------------------------------------
	--------------------------------------------------------
	--------------------------------------------------------
	--FUNCTIONS--
	--------------------------------------------------------
	function spinObjectAtGoldenAngle LastSpinObject GoldSpinPrefs =
	(
		with redraw off
		with undo off
		(

			if undefined != LastSpinObject and isValidNode LastSpinObject then 
			(
				
				if true == JustChangePositions then UpdatedSpinPosAndScale=#()
					else automat__GoldSpinObjArray=#() --This only stores the last operation
				
				local doscale = true
				automat_SpinObj = LastSpinObject
				automat_SpinObjInfoArray[1] = automat_SpinObj.pos
				automat_SpinObjInfoArray[2] = automat_SpinObj.scale
				automat_SpinObjInfoArray[3] = automat_SpinObj.rotation
				
				automat_UserValue = GoldSpinPrefs.UserValue
				
				CenterOfSpin = automat_SpinObj.pos
				randomthreshold = GoldSpinPrefs.BuildPercent
				automat_maxcount = GoldSpinPrefs.Count
				spinangle = GoldSpinPrefs.GAngle
				scalestart = ( 100 - GoldSpinPrefs.Growth ) / 100.0
				padamount = GoldSpinPrefs.Padding / 100.0
				padstart = GoldSpinPrefs.PadStart
				tiltangle = GoldSpinPrefs.Tilt / 10
				dothin = GoldSpinPrefs.GThin
				spinthinamount = GoldSpinPrefs.ThinAmount / 100.0
				
				automat_mode = GoldSpinPrefs.DupMode
				

				--------------
				seed = GoldSpinPrefs.SeedNum
				
				centerthin = if true == dothin and 100 != randomthreshold then ( automat_maxcount * spinthinamount ) as integer else 0
				
				automat_zsize = abs ( automat_SpinObj.max.z - automat_SpinObj.min.z )
				zdrop = GoldSpinPrefs.ZDrop / 1000.0 * automat_zsize
				
				local scaleamount = scalestart
				
				if scalestart != 1 then 
				(
					scaleincrement = ( 1 - scalestart ) * 2
					doscale = true	
				)
				else 
				(
					scaleincrement = 0
					doscale = false
				)
				--------------
				automat_SpinObj.scale = [ scaleamount, scaleamount, scaleamount ]	
				--------------
				automat_tempcenterobj = sphere position:CenterOfSpin
				temprot = automat_tempcenterobj.rotation
				--------------
				automat_xsize = abs ( automat_SpinObj.max.x - automat_SpinObj.min.x )
				automat_ysize = abs ( automat_SpinObj.max.y - automat_SpinObj.min.y )
				currentradius = ( automat_ysize / 10 + automat_xsize / 10 ) * padstart
				--
				automat_RadiusInc =  automat_ysize * padamount --could be made to recalculate inside loop
				--------------------------------------------------------
				move automat_SpinObj [0,currentradius,0]
				automat_SpinObj.parent = automat_tempcenterobj
				
				--------------
				NumCount = 0
				--------------
			
				--------------
				while NumCount < automat_maxcount do
				(
					--------------
					percentofloop = ( ( NumCount as float ) / ( automat_maxcount - 1 ) ) --normalize this number
					easecurveall = ( 3.0 * pow percentofloop 2 ) - ( 2.0 * pow percentofloop 3 )
					easecurve =  ( easecurveall * GoldSpinPrefs.EaseAmount ) + ( ( 1 - GoldSpinPrefs.EaseAmount ) *  percentofloop )
					--------------

						if tiltangle != 0 do
						(
							p = quat ( tiltangle * easecurve )  x_axis
							
							in coordsys automat_SpinObj rotate automat_SpinObj p
						)
					
					
						q = quat spinangle z_axis
						in coordsys local
						(
							rotate automat_tempcenterobj q
						)
						
						--create instance at current location?
						--------------	
						dopercent = ( random 1 100 ) + centerthin
						--format "centerthin: %\n" centerthin
						if centerthin > 0 do centerthin -= 1
						if dopercent <= randomthreshold do 
						(
							if true == JustChangePositions then
							(
								temptransform=#()
								if isValidNode automat_SpinObj do
								(
									append temptransform automat_SpinObj.position
									append temptransform automat_SpinObj.scale
									append UpdatedSpinPosAndScale temptransform
								)
							)
							else
							(
								if 1 == automat_mode then automat_NewObj = instance automat_SpinObj
									else automat_NewObj = copy automat_SpinObj
								
								--------------------------------------------------------
								if false == automat_redraw do flagForeground automat_NewObj true
								--------------------------------------------------------
								automat_NewObj.parent = undefined
								--------------------------------------------------------
								append automat__GoldSpinObjArray automat_NewObj
							)
						)
						--------------
					
						--------------
							
							if true == doscale do 
							(	
								scaleamount = ( scalestart + ( scaleincrement * easecurve ) )
								automat_SpinObj.scale = [ scaleamount, scaleamount, scaleamount ]						
							)
							
							move automat_SpinObj [ 0, 0, ( zdrop * easecurve ) ]

-- 							automat_RadiusInc =  automat_ysize * padamount --@
							currentradius += automat_RadiusInc
							in coordsys local move automat_SpinObj [0,automat_RadiusInc,0]

							
						--NEW. at this, do that
						--/////////////////////////////////////////////
						if GoldSpinPrefs.UseCustom do
						(
							if percentofloop > GoldSpinPrefs.StartCustom do
							(
								num = ( GoldSpinPrefs.StartCustom * automat_maxcount ) as integer
								automat_count = ( NumCount - num + 1 )  --counter
								automat_loop = automat_count as float / (automat_maxcount - num + 1)
								automat_ease = ( 3.0 * pow automat_loop 2 ) - ( 2.0 * pow automat_loop 3 )
-- 								format "automat_maxcount: %\n" automat_maxcount
-- 								format "num: %\n" num
-- 								format "automat_count: %\n" automat_count
-- 								format "automat_loop: %\n" automat_loop
-- 								format "automat_ease: %\n" automat_ease
								execute (  GoldSpinPrefs.Expression  )
								--automat_SpinObj.scale = [ .2, .2, .2 ]		
							)
							
						)
						--------------
						NumCount += 1
						--------------				
				) --END WHILE loop
				
				automat_tempcenterobj.position = CenterOfSpin
				automat_tempcenterobj.rotation = temprot
				automat_tempcenterobj.scale = [1,1,1]
				automat_SpinObj.parent = undefined
				delete automat_tempcenterobj
				automat_SpinObj.rotation = automat_SpinObjInfoArray[3]
				automat_SpinObj.scale = automat_SpinObjInfoArray[2]		
				automat_SpinObj.pos = automat_SpinObjInfoArray[1]
				
				redrawViews()
			)
			else 
			(
				messagebox "No Scatter Object selected or previous object was deleted."
				JustChangePositions = false
			)
			
		)--UNDO OFF
		if true == JustChangePositions do with undo off
		(
			loopmax = UpdatedSpinPosAndScale.count
			for loop = 1 to loopmax do
			(
				if isValidNode automat__GoldSpinObjArray[loop] do 
				(
					automat__GoldSpinObjArray[loop].position = UpdatedSpinPosAndScale[loop][1]
					automat__GoldSpinObjArray[loop].scale = UpdatedSpinPosAndScale[loop][2]
				)
			)
		)
		
	)
	
	
		
	

	--------------------------------------------------------
	function deleteLastGoldSpins =
	(
		for each in automat__GoldSpinObjArray do if isValidNode each do delete each
		automat__GoldSpinObjArray=undefined
		automat__GoldSpinObjArray=#()
		GoldSpinPrefs = undefined
	)

	--------------------------------------------------------
	function selectLastGoldSpins =
	(
		--for piggy in automat__GoldSpinObjArray do try ( delete piggy ) catch()
		clearSelection()
		try ( select automat__GoldSpinObjArray)
			catch
			(
				with redraw off
				(
					clearSelection()
					SelectionArray=#()
					
					max create mode
					for Each in automat__GoldSpinObjArray do
					(
						if true == ( isValidNode  Each ) do append SelectionArray Each
					)
					select SelectionArray
				)
			)
	)

	--------------------------------------------------------
	function selectEveryNGoldSpins EveryN =
	(
		if 0 == EveryN do 
		(
			ClearSelection()
			return()
		)
		with redraw off
		(
			clearSelection()
			max create mode
			
			SelctEveryNArray=#()
			CurrCount = 1
			TotalCount = automat__GoldSpinObjArray.count
			while CurrCount <= TotalCount do
			(
				if mod  CurrCount EveryN == 0 do
				(
					CheckObject = automat__GoldSpinObjArray[CurrCount] 
					if true == ( isValidNode  CheckObject ) do append SelctEveryNArray CheckObject
				)
				CurrCount += 1
			)
			if SelctEveryNArray.count != 0 do select SelctEveryNArray
		)	
	)

	--------------------------------------------------------
	function collpaseLastGoldSpins =
	(
		with undo off
		with redraw off
		(
			restoremode = false
			if getCommandPanelTaskMode() == #modify do
			(
				restoremode = true
				max create mode
			)
			
			arraycount = automat__GoldSpinObjArray.count
		
			automat_SpinObj = ConvertToPoly (  box position:CenterOfSpin name:( "CollapsedSpinObject_000") )
			polyop.deleteFaces automat_SpinObj  #{1..automat_SpinObj.GetNumFaces()}
			
			for loop = 1 to arraycount do
			(
				if isValidNode automat__GoldSpinObjArray[loop] do
				(
					polyop.attach automat_SpinObj automat__GoldSpinObjArray[loop]	
				)
			)
			automat_SpinObj.pivot = CenterOfSpin
			clearSelection()
			select automat_SpinObj
			automat__GoldSpinObjArray=#()
			
			if restoremode do max modify mode
	
		)
	)
	
	--------------------------------------------------------
	function deleteAndSpinAgain =
	(
		for each in automat__GoldSpinObjArray do if isValidNode each do delete each
		automat__GoldSpinObjArray=#()
		JustChangePositions =false
		automat_rebuild = false
		spinObjectAtGoldenAngle LastSpinObject GoldSpinPrefs
	)
	
	--------------------------------------------------------
	function dontDeleteAndSpinAgain =
	(
		JustChangePositions = true
		spinObjectAtGoldenAngle LastSpinObject GoldSpinPrefs
		JustChangePositions = false
	)
	
	--------------------------------------------------------
	function unflagSpinNodes =
	(
		for each in automat__GoldSpinObjArray do if isValidNode each do flagForeground each false
	)
	
	--------------------------------------------------------
	function setRootRenderState state =
	(
		try with undo off
		(
				if 1 == state then
			(
				LastSpinObject.xray = false
				LastSpinObject.renderable = false	
			)
			else
			(
				LastSpinObject.xray = true
				LastSpinObject.renderable = true	
			)
		)
		catch()
		
	)
	
	function getCategoriesFromIniFile =
	(
		--load up ini file and get number and names of all categories within
		--using a counter because it will probably make a cleaner ini file (human readable)
		--I'll need do a counter for the keys anyway so this kinda makes sense here as well
		
		catcounter = 1
		AllCatsArray=#() --RESET this
		getapref = 666 --arbitrary value to get started
		
		while getapref != "" do
		(
		getapref = getINISetting CurrentINI "Categories" ( catcounter as string )
		AllCatsArray[catcounter] = getapref
		catcounter+=1
		)
		
		--Delete the last item in the array if it is null. Faster to do this once at the end
		--then to check for a null each time through the loop above
		if getapref == "" do deleteItem AllCatsArray ( catcounter-1 )
			
		--format "AllCatsArray: %\n" AllCatsArray
		
		-- NOW let's load keys for each category
		--CatCount has how we track how many categories there are
		CatCount = AllCatsArray.count
		--format "CatCount: %\n" CatCount
		
		
			
		roll_automat_SpinScatter.drop_SettingsName.items = AllCatsArray
		
		--now gots to find what category number is currently selected
		--first we look for info in the ini file and make sure it fits i.e. not too big, for the array of categories
		getapref = execute ( getINISetting CurrentINI "User Settings" "Category Selected Item Number" )
		
		if getapref != OK then 
		(
			CurrCat = getapref

		)
		else CurrCat = 1 
		
		--error check the number we get
		if CurrCat > CatCount then CurrCat = 1
		if CatCount == 0 then CurrCat = 0
		
			
		roll_automat_SpinScatter.drop_SettingsName.selection = CurrCat
		
		if CatCount > 0 then
		(
			--CATS are good let's turn to keys
			--LOOP thru the categories
			for loop = 1 to CatCount do
			(
				getapref = ( execute ( getINISetting CurrentINI AllCatsArray[loop]  "keys"  ) )
				
				if getapref != OK then KeyCountArray[loop] = getapref
					else KeyCountArray[loop] = 0
				
			)
			
			--This is the initial setting for the variable
			--this sets the value to the highest key number in the current category
			CurrKey = KeyCountArray[CurrCat] 
			--format "CurrKey on load: %\n" CurrKey
			
			--format "KeyCountArray on load: %\n\n" KeyCountArray
			
			--so we have loaded all categories and counted the keys in each category 
			--we still need to load the data from the file
			
			
		
		
		) 
	) --END Function
	
	function loadSettingsForThisCatAndKey cat key =
	(
		---OK, All cats and keys should be loaded into arrays and the Current Cat and Current Key values have been set.
		--outta here
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".count" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_Count.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Growth" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_Growth.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Padding" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_Padding.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".GAngle" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_Angle.value  = getapref
			
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".PadStart" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_PadStart.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".ZDrop" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_ZDrop.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".BuildPercent" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_Percent.value = getapref
			
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Tilt" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_Tilt.value = getapref
			
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".GThin" ) )
		if getapref != OK do
			roll_automat_SpinScatter.check_spinThin.checked = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".ThinAmount" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_ThinAmount.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".SeedNum" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_Seed.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".UseCustom" ) )
		if getapref != OK do
			roll_automat_SpinScatter.check_UseCustom.checked = getapref

		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".StartCustom" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_StartCustom.value = getapref
		
		getapref =getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Expression" )
		if getapref != "" do
			roll_automat_SpinScatter.combo_Expression.text = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".UserValue" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_UserValue.value = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".DupMode" ) )
		if getapref != OK do
			roll_automat_SpinScatter.radio_DupMode.state = getapref

		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".RenderRoot" ) )
		if getapref != OK do
			roll_automat_SpinScatter.radio_RenderRoot.state = getapref
		
		getapref = execute ( getINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".EaseAmount" ) )
		if getapref != OK do
			roll_automat_SpinScatter.spin_EaseAmount.value = getapref
		
		roll_automat_SpinScatter.getUISettings()
		
		)

		
		
	
		


		function saveIniSettings =
		(
			setapref =setINISetting CurrentINI "User Settings" "Category Selected Item Number" ( CurrCat as string )
			
			--delete previous categories
			delapref =  delINISetting CurrentINI "Categories"
				
			if CatCount > 0 then for loop = 1 to CatCount do
			(
				setapref =  setINISetting CurrentINI "Categories" ( loop as string ) ( AllCatsArray[loop] )
			)
			


		)
		
		function updateSettingsUI cat key resetbool =
		(
			
			
			--is UI active or not?
			if cat > 0 then 
			(
				--UI is active because there is at least one category
				if 1 == cat do
				(
					roll_automat_SpinScatter.drop_SettingsName.enabled = true
					roll_automat_SpinScatter.but_KeyFrame.enabled = true
					roll_automat_SpinScatter.chb_LoadCurrent.enabled = true
					roll_automat_SpinScatter.chb_Lock.enabled = true
					roll_automat_SpinScatter.but_KeyLeft.enabled = true
					roll_automat_SpinScatter.edit_CurrentFrame.enabled = true
					roll_automat_SpinScatter.but_KeyRight.enabled = true
				)
				
				
				if KeyCountArray[cat] != undefined and KeyCountArray[cat]  > 0 then roll_automat_SpinScatter.but_DelCurrent.enabled = true
				else roll_automat_SpinScatter.but_DelCurrent.enabled = false
				
				
				if cat != 0 then roll_automat_SpinScatter.but_DelCategory.enabled = true
					else roll_automat_SpinScatter.but_DelCategory.enabled = false
				
				
				
				--UPDATE UI as needed
				
				--Do we have any keys?
				getapref = ( execute ( getINISetting CurrentINI AllCatsArray[cat]  "keys"  ) )
				
				if getapref != OK then KeyCountArray[cat] = getapref
					else KeyCountArray[cat] = 0
				
				--only want to reset the currkey if a new box was chosen
				if true == resetbool do CurrKey = KeyCountArray[cat]
					
				
				
			
				
				--format "key - read value from ini: %\n" key
				
				if CurrKey < 2 then roll_automat_SpinScatter.but_KeyLeft.enabled = false
					else roll_automat_SpinScatter.but_KeyLeft.enabled = true
				
				if CurrKey == KeyCountArray[cat] then roll_automat_SpinScatter.but_KeyRight.enabled = false
					else roll_automat_SpinScatter.but_KeyRight.enabled = true
				
			)
			--CurrCat is 0. There are no categoris.
			else
			(
				--turn off most of interface because there are no categories
				--only 'new' button remains active
				
				--turnSettingsInterfaceOffOn "no categories"
				
			roll_automat_SpinScatter.drop_SettingsName.enabled = false
			roll_automat_SpinScatter.but_KeyFrame.enabled = false
			roll_automat_SpinScatter.chb_LoadCurrent.enabled = false
			roll_automat_SpinScatter.chb_Lock.enabled = false
			roll_automat_SpinScatter.but_KeyLeft.enabled = false
			
			roll_automat_SpinScatter.edit_CurrentFrame.enabled = false
			roll_automat_SpinScatter.but_KeyRight.enabled = false
			
			roll_automat_SpinScatter.lab_DeleteStuff.enabled = false
			
			roll_automat_SpinScatter.but_DelCurrent.enabled = false
			roll_automat_SpinScatter.but_DelCategory.enabled = false
			
			)
			
			
			--update dropdown list
			
			roll_automat_SpinScatter.drop_SettingsName.items = AllCatsArray
			
			roll_automat_SpinScatter.drop_SettingsName.selection = cat
			
			roll_automat_SpinScatter.edit_CurrentFrame.text = CurrKey as string
			
			
			--going to update keys for current category selection
		)
		
		function storeSettings CurrCat CurrKey =
		(
			--First get current settings
			--this function lives inside the rollout
			roll_automat_SpinScatter.getUISettings()
			
			
			--store key in Array
			--AllKeys_MD_Array [CurrCat] [CurrKey] = GoldSpinPrefs
			
			-- and write number of keys to ini file
			saveKeyNumber CurrCat


			--don't know how to reliably get this to export automagically
			--so here is what manual looks like
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".count" )  ( GoldSpinPrefs.Count as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Growth" )  ( GoldSpinPrefs.Growth as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Padding" )  ( GoldSpinPrefs.Padding as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".GAngle" )  ( GoldSpinPrefs.GAngle as string )
			
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".PadStart" )  ( GoldSpinPrefs.PadStart as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".ZDrop" )  ( GoldSpinPrefs.ZDrop as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".BuildPercent" )  ( GoldSpinPrefs.BuildPercent as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Tilt" )  ( GoldSpinPrefs.Tilt as string )
			
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".GThin" )  ( GoldSpinPrefs.GThin as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".ThinAmount" )  ( GoldSpinPrefs.ThinAmount as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".SeedNum" )  ( GoldSpinPrefs.SeedNum as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".UseCustom" )  ( GoldSpinPrefs.UseCustom as string )

			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".StartCustom" )  ( GoldSpinPrefs.StartCustom as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".Expression" )  ( GoldSpinPrefs.Expression as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".UserValue" )  ( GoldSpinPrefs.UserValue as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".DupMode" )  ( GoldSpinPrefs.DupMode as string )

			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".RenderRoot" )  ( GoldSpinPrefs.RenderRoot as string )
			setapref =setINISetting CurrentINI AllCatsArray[CurrCat]  ( CurrKey as string + ".EaseAmount" )  ( GoldSpinPrefs.EaseAmount as string )


		)
		
		function deleteSettings CurrCat CurrKey keyorcat =
		(
			--want to declare these temp variables, they assume we are deleting a single key
			loop = 1
			loopstart = CurrKey
			loopend = CurrKey
			thecat = AllCatsArray[CurrCat]
			
			--first determine whether we are deleting a key or a category
			if "key" == keyorcat  then
			(
				--decrement key count and save keys
				if CurrKey >0 do
				(
					CurrKey -= 1
					KeyCountArray[CurrCat] -= 1
				)
				
				
				saveKeyNumber CurrCat
				
				
				--
				
				for loop = loopstart to  loopend do
				(
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".count" ) 
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".Growth" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".Padding" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".GAngle" )
					
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".PadStart" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".ZDrop" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".BuildPercent" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".Tilt" )
					
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".GThin" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".ThinAmount" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".SeedNum" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".UseCustom" )

					delapref =delINISetting CurrentINI thecat  ( loop as string + ".StartCustom" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".Expression" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".UserValue" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".DupMode" )

					delapref =delINISetting CurrentINI thecat  ( loop as string + ".RenderRoot" )
					delapref =delINISetting CurrentINI thecat  ( loop as string + ".EaseAmount" )
				)
				
			)
			else
			(
				--we are gonna remove the whole category
				--confirm dialog
				--set loop start and end from number of keys
				--delete category entry
				--delete category key entry
				--delete all keys using for loop
				
				if true == ( queryBox "This will delete the current category and all stored settings within. Are you sure?" ) then
				(

					--delete the keys for the current cateogry
					--no key sepcified so the entire category goes bye bye
					--/////////////////////////////////////////////////////
					--//////////////////////////////////////////////////////
					--CRASH when CurrKey often hits an illegal value one over what it should be
					delapref =delINISetting CurrentINI AllCatsArray[CurrCat]
					
					--
					
					--this should delete the current category from the allcats array
					deleteItem AllCatsArray ( CurrCat )
					
					--this should delete the entry for the current category in the keycount array
					--deleteItem KeyCountArray ( CurrCat )

					--
					CatCount -= 1
						CurrCat = CatCount

					--
					
					--dave current categories
					saveIniSettings()
					
					
					
				)
				else
				(
					--do nothing
					return()
				)
			)
			


			
			
			--after loop 
			--reload settings from ini, otherwise things just get ugly
			roll_automat_SpinScatter.edit_CurrentFrame.text = "-"
			getCategoriesFromIniFile()
			updateSettingsUI CurrCat CurrKey true


		)
		
		function saveKeyNumber CurrCat =
		(
			
			
			
			
			setapref = setINISetting CurrentINI AllCatsArray[CurrCat]  "keys"  ( KeyCountArray[CurrCat] as string )
		)
		
		function turnSettingsInterfaceOffOn dostring =
		(
			if "no categories" == dostring do
			(
				--roll_automat_SpinScatter.drop_SettingsName.enabled = false
-- 				roll_automat_SpinScatter.drop_SettingsName.enabled = false
-- 				roll_automat_SpinScatter.drop_SettingsName.enabled = false
-- 				roll_automat_SpinScatter.drop_SettingsName.enabled = false
-- 				roll_automat_SpinScatter.drop_SettingsName.enabled = false
-- 				
				
			)
			
		)
		
		function editCategoryInfo  windowtitle textstring  EditCat_RenameMode =
	(
		try( closeRolloutfloater automat_EditCatName ) catch()
		
		
		getapref = execute ( getINISetting CurrentINI "Edit Categroy Rollout" "RolloutPosition" )
			if getapref != OK do EditCat_RolloutPosition = getapref
				
		--
			
		automat_EditCatName = newRolloutFloater EditCat_WindowTitle 200 103  EditCat_RolloutPosition.x EditCat_RolloutPosition.y
		addRollout roll_automat_EditCatName automat_EditCatName
	
	)
	
	function renameIniSettings oldname getthetext =
	(
		messagebox ( "Update your ini file here: " + CurrentINI as string )
-- 		--category number is the current category, getthetext is the new username
-- 		--find all keys under original name
-- 		--copy
-- 		
-- 		superhacky = openFile CurrentINI mode:"r+"
-- 		searchstring = "[" + oldname + "]"
-- 		replacestring = "[" + getthetext + "]"
-- 		thecount = searchstring.count
-- 		
-- 		skipToString superhacky searchstring
-- 		
-- 		beeper = filePos superhacky
-- 		
-- 		flush superhacky
-- 		
-- 		seek superhacky beeper
-- 		
-- 		print replacestring to:superhacky
-- 		
-- 		close superhacky
-- 		--delete category now
	)
			
	
	
	if true != automat_OpenScriptAsRollout do
	(
		try (closeRolloutfloater automat_SpinScatter ) catch ()
		automat_SpinScatter = newRolloutFloater "Spin Scatter" 170 200 RolloutPosition.x RolloutPosition.y
		addRollout roll_automat_SpinScatter automat_SpinScatter
		automat_SpinScatter.size.y = ( roll_automat_SpinScatter.height + 30 )
	)
	


	
)
